#ifndef _INCLUDED_UTILS_H_
#define _INCLUDED_UTILS_H_

#include "types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _BYTE_ARRAY
{
	uint8 *buffer;
	size_t num_data;
	size_t buffer_size;
	size_t block_size;
} BYTE_ARRAY;

typedef struct _WORD_ARRAY
{
	uint16 *buffer;
	size_t num_data;
	size_t buffer_size;
	size_t block_size;
} WORD_ARRAY;

typedef struct _UINT32_ARRAY
{
	uint32 *buffer;
	size_t num_data;
	size_t buffer_size;
	size_t block_size;
} UINT32_ARRAY;

typedef struct _POINTER_ARRAY
{
	void **buffer;
	size_t num_data;
	size_t buffer_size;
	size_t block_size;
} POINTER_ARRAY;

typedef struct _STRUCT_ARRAY
{
	uint8 *buffer;
	size_t num_data;
	size_t buffer_size;
	size_t block_size;
	size_t data_size;
} STRUCT_ARRAY;

/**************************************
* StringCompareIgnoreCase֐         *
* 啶/𖳎ĕr *
*                                 *
* str1	: r1                 *
* str2	: r2                 *
* Ԃl                              *
*	̍(Ȃ0)         *
**************************************/
EXTERN int StringCompareIgnoreCase(const char* str1, const char* str2);

/**************************************
* memset32֐                        *
* 32bitȂŃobt@𖄂߂ *
*                                 *
* buff	: ߂Ώۂ̃obt@        *
* value	: ߂l                    *
* size	: ߂oCg              *
**************************************/
EXTERN void memset32(void* buff, uint32 value, size_t size);

/**************************************
* memset64֐                        *
* 64bitȂŃobt@𖄂߂ *
*                                 *
* buff	: ߂Ώۂ̃obt@        *
* value	: ߂l                    *
* size	: ߂oCg              *
**************************************/
EXTERN void memset64(void* buff, uint64 value, size_t size);

EXTERN BYTE_ARRAY* ByteArrayNew(size_t block_size);
EXTERN void ByteArrayAppend(BYTE_ARRAY* barray, uint8 data);
EXTERN void ByteArrayDesteroy(BYTE_ARRAY** barray);
EXTERN void ByteArrayResize(BYTE_ARRAY* barray, size_t new_size);

EXTERN WORD_ARRAY* WordArrayNew(size_t block_size);
EXTERN void WordArrayAppend(WORD_ARRAY* warray, uint16 data);
EXTERN void WordArrayDestroy(WORD_ARRAY** warray);
EXTERN void WordArrayResize(WORD_ARRAY* warray, size_t new_size);

EXTERN UINT32_ARRAY* Uint32ArrayNew(size_t block_size);
EXTERN void Uint32ArrayAppend(UINT32_ARRAY* uarray, uint32 data);
EXTERN void Uint32ArrayDestroy(UINT32_ARRAY** uarray);
EXTERN void Uint32ArrayResize(UINT32_ARRAY* uint32_array, size_t new_size);

EXTERN POINTER_ARRAY* PointerArrayNew(size_t block_size);
EXTERN void PointerArrayRelease(
	POINTER_ARRAY* pointer_array,
	void (*destroy_func)(void*)
);
EXTERN void PointerArrayDestroy(
	POINTER_ARRAY** pointer_array,
	void (*destroy_func)(void*)
);
void PointerArrayAppend(POINTER_ARRAY* pointer_array, void* data);
EXTERN void PointerArrayRemoveByIndex(
	POINTER_ARRAY* pointer_array,
	size_t index,
	void (*destroy_func)(void*)
);
EXTERN void PointerArrayRemoveByData(
	POINTER_ARRAY* pointer_array,
	void* data,
	void (*destroy_func)(void*)
);
EXTERN int PointerArrayDoesCointainData(POINTER_ARRAY* pointer_array, void* data);

EXTERN STRUCT_ARRAY* StructArrayNew(size_t data_size, size_t block_size);
EXTERN void StructArrayDestroy(
	STRUCT_ARRAY** struct_array,
	void (*destroy_func)(void*)
);
EXTERN void StructArrayAppend(STRUCT_ARRAY* struct_array, void* data);
EXTERN void* StructArrayReserve(STRUCT_ARRAY* struct_array);
EXTERN void StructArrayResize(STRUCT_ARRAY* struct_array, size_t new_size);
EXTERN void StructArrayRemoveByIndex(
	STRUCT_ARRAY* struct_array,
	size_t index,
	void (*destroy_func)(void*)
);

#ifdef _MSC_VER
EXTERN int strncasecmp(const char* s1, const char* s2, size_t n);
#endif

/**************************************************************
* StringStringIgnoreCase֐                                  *
* 啶/𖳎Ĉ1̕񂩂2̕T *
* str		: Ώۂ̕                                *
* search	: 镶                                  *
* Ԃl                                                      *
*	񔭌:ʒũ|C^	Ȃ:NULL     *
**************************************************************/
EXTERN char* StringStringIgnoreCase(const char* str, const char* search);

/*********************************
* GetFileExtention֐           *
* t@Cgq擾 *
*                            *
* file_name	: t@C         *
* Ԃl                         *
*	gq̕               *
*********************************/
EXTERN char* GetFileExtention(char* file_name);

/***************************************
* StringReplace֐                    *
* w肵u             *
*                                  *
* str			: 镶     *
* replace_from	: u镶 *
* replace_to	: u镶     *
***************************************/
EXTERN void StringRepalce(
	char* str,
	char* replace_from,
	char* replace_to
);

EXTERN int ForFontFamilySearchCompare(const char* str, PangoFontFamily** font);

/*******************************************
* FileRead֐                             *
* Gtk̊֐𗘗păt@Cǂݍ      *
*                                      *
* dst			: ǂݍݐ̃AhX     *
* block_size	: ǂݍރubÑTCY *
* read_num		: ǂݍރubN̐     *
* stream		: ǂݍ݌̃Xg[   *
* Ԃl                                   *
*	ǂݍ񂾃ubN                   *
*******************************************/
EXTERN size_t FileRead(void* dst, size_t block_size, size_t read_num, GFileInputStream* stream);

/*******************************************
* FileWrite֐                            *
* Gtk̊֐𗘗păt@C      *
*                                      *
* src			: ݌̃AhX     *
* block_size	: ރubÑTCY *
* read_num		: ރubN̐     *
* stream		: ݐ̃Xg[   *
* Ԃl                                   *
*	񂾃ubN                   *
*******************************************/
EXTERN size_t FileWrite(void* src, size_t block_size, size_t read_num, GFileOutputStream* stream);

/********************************************
* FileSeek֐                              *
* Gtk̊֐𗘗păt@CV[N         *
*                                       *
* stream	: V[NsXg[        *
* offset	: ړoCg                  *
* origin	: ړJnʒu(fseek֐Ɠ) *
* Ԃl                                    *
*	I(0), ُI(0ȊO)            *
********************************************/
EXTERN int FileSeek(void* stream, long offset, int origin);

/************************************************
* FileSeekTell֐                              *
* Gtk̊֐𗘗păt@C̃V[NʒuԂ *
*                                           *
* stream	: V[Nʒu𒲂ׂXg[      *
* Ԃl                                        *
*	V[Nʒu                                  *
************************************************/
EXTERN long FileSeekTell(void* stream);

/***********************************************
* InvertMatrix֐                             *
* tsvZ                             *
*                                          *
* a	: vZΏۂ̍s(tsf[^͂ɓ) *
* n	: s̎                               *
***********************************************/
EXTERN void InvertMatrix(FLOAT_T **a, int n);

/*********************************
* FLAG_CHECK}N֐           *
* tOON/OFF𔻒肷       *
*                            *
* FLAGS	: tOz             *
* ID	: `FbNtOID *
* Ԃl                         *
*	ON:0ȊO OFF:0               *
*********************************/
#define FLAG_CHECK(FLAGS, ID) (FLAGS[((ID)/8)] & (1 <<((ID)%8)))

/*****************************
* FLAG_ON}N֐          *
* tOONɂ           *
*                        *
* FLAGS	: tOz         *
* ID	: ONɂtOID *
*****************************/
#define FLAG_ON(FLAGS, ID) FLAGS[((ID)/8)] |= (1 << ((ID)%8))

/******************************
* FLAG_OFF}N֐          *
* tOOFFɂ           *
*                         *
* FLAGS	: tOz          *
* ID	: OFFɂtOID *
*****************************/
#define FLAG_OFF(FLAGS, ID) FLAGS[((ID)/8)] &= ~(1 << ((ID)%8))

EXTERN void AdjustmentChangeValueCallBackInt8(GtkAdjustment* adjustment, int8* store);
EXTERN void AdjustmentChangeValueCallBackUint8(GtkAdjustment* adjustment, uint8* store);
EXTERN void AdjustmentChangeValueCallBackInt16(GtkAdjustment* adjustment, int16* store);
EXTERN void AdjustmentChangeValueCallBackUint16(GtkAdjustment* adjustment, uint16* store);
EXTERN void AdjustmentChangeValueCallBackInt32(GtkAdjustment* adjustment, int32* store);
EXTERN void AdjustmentChangeValueCallBackUint32(GtkAdjustment* adjustment, uint32* store);
EXTERN void AdjustmentChangeVaueCallBackDouble(GtkAdjustment* adjustment, gdouble* value);

EXTERN void CheckButtonSetFlagsCallBack(GtkWidget* button, guint32* flags, guint32 flag_value);

/*******************************************************
* InflateData֐                                      *
* ZIPkꂽf[^fR[h                    *
*                                                  *
* data				: ̓f[^                       *
* out_buffer		: o͐̃obt@                 *
* in_size			: ̓f[^̃oCg             *
* out_buffer_size	: o͐̃obt@̃TCY         *
* out_size			: o͂oCg̊i[(NULL) *
* Ԃl                                               *
*	I:0As:0ȊO                             *
*******************************************************/
EXTERN int InflateData(
	uint8* data,
	uint8* out_buffer,
	size_t in_size,
	size_t out_buffer_size,
	size_t* out_size
);

/***************************************************
* DeflateData֐                                  *
* ZIPks                                    *
*                                              *
* data					: ̓f[^               *
* out_buffer			: o͐̃obt@         *
* target_data_size		: ̓f[^̃oCg     *
* out_buffer_size		: o͐̃obt@̃TCY *
* compressed_data_size	: k̃oCgi[   *
* compress_level		: kx(0`9)         *
* Ԃl                                           *
*	I:0As:0ȊO                         *
***************************************************/
EXTERN int DeflateData(
	uint8* data,
	uint8* out_buffer,
	size_t target_data_size,
	size_t out_buffer_size,
	size_t* compressed_data_size,
	int compress_level
);

EXTERN void UpdateWidget(GtkWidget* widget);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef _INCLUDED_UTILS_H_
