#include <string.h>
#include "../../plug_in.h"
#include "../../filter.h"
#include "../../spin_scale.h"
#include "../../utils.h"
#include "unsharp_mask.h"

#ifdef __cplusplus
extern "C" {
#endif

void InitializeUnsharpMask(UNSHARP_MASK_DATA* mask_data)
{
#define DEFAULT_RANGE 1
#define DEFAULT_RATE 1.0
	(void)memset(mask_data, 0, sizeof(*mask_data));
	mask_data->range = DEFAULT_RANGE;
	mask_data->rate = DEFAULT_RATE;
}

int UnsharpMask(
	UNSHARP_MASK_DATA* unsharp,
	LAYER* target,
	int (*progress_func)(float rate, void* data),
	void* data
)
{
	LAYER *temp_layer = target->window->temp_layer;
	int diff_color[4];
	int color[4];
	int i;

	(void)memcpy(temp_layer->pixels, target->pixels, target->stride * target->height);

	ApplyBlurFilter(temp_layer, unsharp->range);

	for(i=0; i<target->width*target->height; i++)
	{
		diff_color[0] = temp_layer->pixels[i*4] - target->pixels[i*4];
		diff_color[1] = temp_layer->pixels[i*4+1] - target->pixels[i*4+1];
		diff_color[2] = temp_layer->pixels[i*4+2] - target->pixels[i*4+2];
		diff_color[3] = temp_layer->pixels[i*4+3] - target->pixels[i*4+3];

		color[0] = (int)(target->pixels[i*4] - diff_color[0]*unsharp->rate);
		color[1] = (int)(target->pixels[i*4+1] - diff_color[1]*unsharp->rate);
		color[2] = (int)(target->pixels[i*4+2] - diff_color[2]*unsharp->rate);
		color[3] = (int)(target->pixels[i*4+3] - diff_color[3]*unsharp->rate);

		if(color[0] < 0)
		{
			color[0] = 0;
		}
		else if(color[0] > 255)
		{
			color[0] = 255;
		}

		if(color[1] < 0)
		{
			color[1] = 0;
		}
		else if(color[1] > 255)
		{
			color[1] = 255;
		}

		if(color[2] < 0)
		{
			color[2] = 0;
		}
		else if(color[2] > 255)
		{
			color[2] = 255;
		}

		if(color[3] < 0)
		{
			color[3] = 0;
		}
		else if(color[3] > 255)
		{
			color[3] = 255;
		}

		if(color[0] > color[3])
		{
			color[0] = color[3];
		}
		if(color[1] > color[3])
		{
			color[1] = color[3];
		}
		if(color[2] > color[3])
		{
			color[2] = color[3];
		}

		target->pixels[i*4] = color[0];
		target->pixels[i*4+1] = color[1];
		target->pixels[i*4+2] = color[2];
		target->pixels[i*4+3] = color[3];
	}

	return 0;
}

GtkWidget* UnsharpMaskSettingWidgetNew(UNSHARP_MASK_DATA* unsharp, APPLICATION* app)
{
	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
	GtkWidget *control;
	GtkAdjustment *adjustment;

	adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(unsharp->range,
		1, 30, 1, 1, 0));
	AdjustmentChangeValueCallBackInt(adjustment, &unsharp->range);
	control = SpinScaleNew(adjustment, app->labels->tool_box.scale, 0);
	gtk_box_pack_start(GTK_BOX(vbox), control, FALSE, FALSE, 0);

	adjustment = GTK_ADJUSTMENT(gtk_adjustment_new(unsharp->rate,
		0.1, 10, 0.1, 0.1, 0));
	AdjustmentChangeValueCallBackDouble(adjustment, &unsharp->rate);
	control = SpinScaleNew(adjustment, "Rate", 1);
	gtk_box_pack_start(GTK_BOX(vbox), control, FALSE, FALSE, 0);

	return vbox;
}

#ifdef __cplusplus
}
#endif
