// Visual Studio 2005ȍ~ł͌ÂƂ֐gp̂
	// xoȂ悤ɂ
#if defined _MSC_VER && _MSC_VER >= 1400
# define _CRT_SECURE_NO_DEPRECATE
#endif

#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "../../plug_in.h"
#include "../../utils.h"
#include "../../image_read_write.h"
#include "../../memory_stream.h"

typedef enum _eMINE_SWEEPER_DIFFICULTY
{
	MINE_SWEEPER_DIFFICULTY_EASY,
	MINE_SWEEPER_DIFFICULTY_MIDIUM,
	MINE_SWEEPER_DIFFICULTY_HARD
} eMINE_SWEEPER_DIFFICULTY;

typedef enum _eMINE_SWEEPER_FLAGS
{
	MINE_SWEEPER_FLAG_STARTED = 0x01,
	MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED = 0x02,
	MINE_SWEEPER_FLAG_GAME_OVER = 0x04,
	MINE_SWEEPER_FLAG_CLEARED = 0x08
} eMINE_SWEEPER_FLAG_;

typedef struct _MINE_SWEEPER
{
	eMINE_SWEEPER_DIFFICULTY difficulty;
	int width, height;
	uint8 *opened;
	uint8 *mine;
	uint8 *flaged;
	uint8 **num_mine;
	int mine_count;
	cairo_surface_t *mine_surface;
	cairo_surface_t *open_tile_surface;
	cairo_surface_t *close_tile_surface;
	cairo_surface_t *flag_surface;
	int tile_width, tile_height;
	int num_opened;
	unsigned int flags;
	APPLICATION *application;
	GModule *module;
	GtkWidget *window;
	GtkWidget *canvas;
	GtkWidget *clear_label;
} MINE_SWEEPER;

static void InitializeBuffer(MINE_SWEEPER* sweeper);
static void InitializeBoard(MINE_SWEEPER* sweeper, int start_x, int start_y);

const uint8 mine_image_data[] =
{
#include "mine.txt"
};

const uint8 flag_image_data[] =
{
#include "flag.txt"
};

void InitializeMineSweeper(MINE_SWEEPER* sweeper)
{
	MEMORY_STREAM stream;
	cairo_surface_t *surface_p;
	cairo_t *cairo_p;
	uint8 *pixels;
	gint32 width, height, stride;
	uint8 r;
	int i;

	(void)memset(sweeper, 0, sizeof(*sweeper));
	InitializeBuffer(sweeper);
	InitializeBoard(sweeper, 0, 0);

	stream.block_size = 1;

	stream.buff_ptr = (uint8*)mine_image_data;
	stream.data_point = 0;
	stream.data_size = sizeof(mine_image_data);
	pixels = ReadPNGStream(&stream, (stream_func_t)MemRead, &width, &height, &stride);
	for(i=0; i<width*height; i++)
	{
		r = pixels[i*4];
		pixels[i*4] = pixels[i*4+2];
		pixels[i*4+2] = r;

		pixels[i*4] = MINIMUM(pixels[i*4], pixels[i*4+3]);
		pixels[i*4+1] = MINIMUM(pixels[i*4+1], pixels[i*4+3]);
		pixels[i*4+2] = MINIMUM(pixels[i*4+2], pixels[i*4+3]);
	}
	sweeper->mine_surface = cairo_image_surface_create_for_data(pixels,
		CAIRO_FORMAT_ARGB32, width, height, stride);
	sweeper->tile_width = width;
	sweeper->tile_height = height;

	stream.buff_ptr = (uint8*)flag_image_data;
	stream.data_point = 0;
	stream.data_size = sizeof(flag_image_data);
	pixels = ReadPNGStream(&stream, (stream_func_t)MemRead, &width, &height, &stride);
	for(i=0; i<width*height; i++)
	{
		r = pixels[i*4];
		pixels[i*4] = pixels[i*4+2];
		pixels[i*4+2] = r;

		pixels[i*4] = MINIMUM(pixels[i*4], pixels[i*4+3]);
		pixels[i*4+1] = MINIMUM(pixels[i*4+1], pixels[i*4+3]);
		pixels[i*4+2] = MINIMUM(pixels[i*4+2], pixels[i*4+3]);
	}
	sweeper->flag_surface = cairo_image_surface_create_for_data(pixels,
		CAIRO_FORMAT_ARGB32, width, height, stride);

	pixels = (uint8*)calloc(1, sweeper->tile_width * sweeper->tile_height * 4);
	surface_p = sweeper->open_tile_surface = cairo_image_surface_create_for_data(pixels,
		CAIRO_FORMAT_ARGB32, sweeper->tile_width, sweeper->tile_height, sweeper->tile_width * 4);
	(void)memset(pixels, 0, sweeper->tile_width * sweeper->tile_height * 4);

	pixels = (uint8*)calloc(1, sweeper->tile_width * sweeper->tile_height * 4);
	surface_p = sweeper->close_tile_surface = cairo_image_surface_create_for_data(pixels,
		CAIRO_FORMAT_ARGB32, sweeper->tile_width, sweeper->tile_height, sweeper->tile_width * 4);
	cairo_p = cairo_create(surface_p);

	cairo_set_line_width(cairo_p, sweeper->tile_width * 0.15);

	cairo_set_source_rgb(cairo_p, 0.6, 0.6, 0.6);
	cairo_rectangle(cairo_p, 0, 0, sweeper->tile_width, sweeper->tile_height);
	cairo_fill(cairo_p);

	cairo_set_source_rgb(cairo_p, 0.3, 0.3, 0.3);
	cairo_move_to(cairo_p, 0, sweeper->tile_height);
	cairo_line_to(cairo_p, sweeper->tile_width, sweeper->tile_width);
	cairo_stroke(cairo_p);
	cairo_move_to(cairo_p, sweeper->tile_width - sweeper->tile_width * 0.02, 0);
	cairo_line_to(cairo_p, sweeper->tile_width - sweeper->tile_width * 0.02, sweeper->tile_height);
	cairo_stroke(cairo_p);

	cairo_set_source_rgb(cairo_p, 0.9, 0.9, 0.9);
	cairo_move_to(cairo_p, sweeper->tile_width * 0.02, 0);
	cairo_line_to(cairo_p, sweeper->tile_width * 0.02, sweeper->tile_height);
	cairo_stroke(cairo_p);
	cairo_move_to(cairo_p, 0, sweeper->tile_height * 0.02);
	cairo_line_to(cairo_p, sweeper->tile_width, sweeper->tile_height * 0.02);
	cairo_stroke(cairo_p);

	cairo_destroy(cairo_p);
}


static void InitializeBuffer(MINE_SWEEPER* sweeper)
{
	const int difficulty_widths[] = {9, 16, 30};
	const int difficulty_heights[] = {9, 16, 16};
	const int difficulty_mines[] = {10, 40, 99};
	const int size = difficulty_widths[sweeper->difficulty] * difficulty_heights[sweeper->difficulty];
	int i;

	sweeper->width = difficulty_widths[sweeper->difficulty];
	sweeper->height = difficulty_heights[sweeper->difficulty];
	sweeper->mine_count = difficulty_mines[sweeper->difficulty];

	sweeper->opened = (uint8*)malloc((size + 7) / 8);
	sweeper->mine = (uint8*)malloc((size + 7) / 8);
	sweeper->flaged = (uint8*)malloc((size + 7) / 8);

	sweeper->num_mine = (uint8**)malloc(sizeof(*sweeper->num_mine) * sweeper->height);
	sweeper->num_mine[0] = (uint8*)malloc(size);
	for(i=1; i<sweeper->height; i++)
	{
		sweeper->num_mine[i] = &sweeper->num_mine[0][i * sweeper->width];
	}
}

static void ReleaseBuffer(MINE_SWEEPER* sweeper)
{
	free(sweeper->opened);
	free(sweeper->mine);
	free(sweeper->flaged);
	free(sweeper->num_mine[0]);
	free(sweeper->num_mine);
}

void ReleaseMineSweeper(MINE_SWEEPER* sweeper)
{
	ReleaseBuffer(sweeper);
	free(cairo_image_surface_get_data(sweeper->open_tile_surface));
	cairo_surface_destroy(sweeper->open_tile_surface);
	free(cairo_image_surface_get_data(sweeper->close_tile_surface));
	cairo_surface_destroy(sweeper->close_tile_surface);
	MemoryFree(cairo_image_surface_get_data(sweeper->mine_surface));
	cairo_surface_destroy(sweeper->mine_surface);
	MemoryFree(cairo_image_surface_get_data(sweeper->flag_surface));
}

static int OpenTile(MINE_SWEEPER* sweeper, int x, int y)
{
	int ret = 0;
	if(x < 0 || x >= sweeper->width || y < 0 || y >= sweeper->height)
	{
		return 0;
	}

	if(FLAG_CHECK(sweeper->opened, y * sweeper->width + x)
		|| FLAG_CHECK(sweeper->flaged, y * sweeper->width + x))
	{
		return 0;
	}

	FLAG_ON(sweeper->opened, y * sweeper->width + x);

	if(FLAG_CHECK(sweeper->mine, y * sweeper->width + x))
	{
		return -1;
	}
	else if(sweeper->num_mine[y][x] == 0)
	{
		ret += OpenTile(sweeper, x-1, y-1);
		ret += OpenTile(sweeper, x, y-1);
		ret += OpenTile(sweeper, x+1, y-1);
		ret += OpenTile(sweeper, x-1, y);
		ret += OpenTile(sweeper, x+1, y);
		ret += OpenTile(sweeper, x-1, y+1);
		ret += OpenTile(sweeper, x, y+1);
		ret += OpenTile(sweeper, x+1, y+1);
	}

	return ret;
}

static int CheckSweeped(MINE_SWEEPER* sweeper)
{
	int x, y;
	for(y=0; y<sweeper->height; y++)
	{
		for(x=0; x<sweeper->width; x++)
		{
			if(!(FLAG_CHECK(sweeper->mine, y*sweeper->width + x))
				&& (!FLAG_CHECK(sweeper->opened, y*sweeper->width + x)))
			{
				return FALSE;
			}
		}
	}

	return TRUE;
}

static void InitializeBoard(MINE_SWEEPER* sweeper, int start_x, int start_y)
{
#define CHECK_CORRECT(X, Y, MINE, START_X, START_Y, WIDTH) ((X) != (START_X) && (Y) != (START_Y) \
		&& (X) - 1 != (START_X) && (Y) - 1 != (START_Y) && (X) + 1 != (START_X) && (Y) + 1 != (START_Y) \
		&& (FLAG_CHECK((MINE), (WIDTH) * (Y) + (X)) == 0))
	const int size = sweeper->width * sweeper->height;
	int x, y;
	int i;

	if(start_x < 0 || start_x >= sweeper->width || start_y < 0 || start_y >= sweeper->height)
	{
		return;
	}

	(void)memset(sweeper->opened, 0, (size + 7) / 8);
	(void)memset(sweeper->mine, 0, (size + 7) / 8);
	(void)memset(sweeper->flaged, 0, (size + 7) / 8);
	(void)memset(sweeper->num_mine[0], 0, size);
	sweeper->num_opened = 0;
	sweeper->flags &= ~(MINE_SWEEPER_FLAG_STARTED | MINE_SWEEPER_FLAG_CLEARED
		| MINE_SWEEPER_FLAG_GAME_OVER);

	srand((int)time(NULL));
	for(i=0; i<sweeper->mine_count; i++)
	{
		do
		{
			x = rand() % sweeper->width;
			y = rand() % sweeper->height;
		} while( ! CHECK_CORRECT(x, y, sweeper->mine, start_x, start_y, sweeper->width));

		FLAG_ON(sweeper->mine, y * sweeper->width + x);
		if(y > 0)
		{
			if(x > 0)
			{
				sweeper->num_mine[y-1][x-1]++;
			}
			sweeper->num_mine[y-1][x]++;
			if(x < sweeper->width - 1)
			{
				sweeper->num_mine[y-1][x+1]++;
			}
		}
		if(x > 0)
		{
			sweeper->num_mine[y][x-1]++;
		}
		if(x < sweeper->width - 1)
		{
			sweeper->num_mine[y][x+1]++;
		}
		if(y < sweeper->height - 1)
		{
			if(x > 0)
			{
				sweeper->num_mine[y+1][x-1]++;
			}
			sweeper->num_mine[y+1][x]++;
			if(x < sweeper->width - 1)
			{
				sweeper->num_mine[y+1][x+1]++;
			}
		}
	}
}

static int CountAroundFlags(MINE_SWEEPER* sweeper, int x, int y)
{
	int ret = 0;

	if(y > 0)
	{
		if(x > 0)
		{
			if(FLAG_CHECK(sweeper->flaged, (y-1)*sweeper->width + (x-1)))
			{
				ret++;
			}
		}
		if(FLAG_CHECK(sweeper->flaged, (y-1)*sweeper->width + x))
		{
			ret++;
		}
		if(x < sweeper->width - 1)
		{
			if(FLAG_CHECK(sweeper->flaged, (y-1)*sweeper->width + (x+1)))
			{
				ret++;
			}
		}
	}

	if(x > 0)
	{
		if(FLAG_CHECK(sweeper->flaged, y*sweeper->width + (x-1)))
		{
			ret++;
		}
	}
	if(x < sweeper->width - 1)
	{
		if(FLAG_CHECK(sweeper->flaged, y*sweeper->width + (x+1)))
		{
			ret++;
		}
	}

	if(y < sweeper->height - 1)
	{
		if(x > 0)
		{
			if(FLAG_CHECK(sweeper->flaged, (y+1)*sweeper->width + (x-1)))
			{
				ret++;
			}
		}
		if(FLAG_CHECK(sweeper->flaged, (y+1)*sweeper->width + x))
		{
			ret++;
		}
		if(x < sweeper->width - 1)
		{
			if(FLAG_CHECK(sweeper->flaged, (y+1)*sweeper->width + (x+1)))
			{
				ret++;
			}
		}
	}

	return ret;
}

static void SetClearLabel(MINE_SWEEPER* sweeper)
{
	char str[512];
	if((sweeper->flags & MINE_SWEEPER_FLAG_CLEARED) != 0)
	{
		(void)sprintf(str, "<span font_desc=\"24\" color=\"red\">Clear ! </span>");
		gtk_label_set_markup(GTK_LABEL(sweeper->clear_label), str);
	}
	else if((sweeper->flags & MINE_SWEEPER_FLAG_GAME_OVER) != 0)
	{
		(void)sprintf(str, "<span font_desc=\"24\" color=\"blue\">Game Over... ! </span>");
		gtk_label_set_markup(GTK_LABEL(sweeper->clear_label), str);
	}
	else
	{
		(void)sprintf(str, "<span font_desc=\"24\"> </span>");
		gtk_label_set_markup(GTK_LABEL(sweeper->clear_label), str);
	}
}

static void OpenAllTile(MINE_SWEEPER* sweeper)
{
	(void)memset(sweeper->opened, 0xff, ((sweeper->width * sweeper->height + 7) / 8));
	SetClearLabel(sweeper);
}

#if GTK_MAJOR_VERSION <= 2
static gboolean DisplayBoard(GtkWidget* widget, GdkEventExpose* event_info, MINE_SWEEPER* sweeper)
#else
static gboolean DisplayBoard(GtkWidget* widget, cairo_t* cairo_p, MINE_SWEEPER* sweeper)
#endif
{
#if GTK_MAJOR_VERSION <= 2
	cairo_t *cairo_p = gdk_cairo_create(event_info->window);
#endif
	GtkAllocation allocation;
	cairo_matrix_t pattern_matrix;
	FLOAT_T x_step,	y_step;
	FLOAT_T tile_scale_x,	tile_scale_y;
	FLOAT_T display_x, display_y;
	FLOAT_T font_scale;
	cairo_pattern_t *open_pattern, *close_pattern;
	cairo_pattern_t *mine_pattern, *flag_pattern;
	cairo_pattern_t *set_pattern;
	cairo_text_extents_t character_info;
	char str[4] = {0};
	int opened;
	int x, y;

#if GTK_MAJOR_VERSION <= 2
	allocation = widget->allocation;
#else
	gtk_widget_get_allocation(widget, &allocation);
#endif

	if(sweeper->application != NULL)
	{
		DRAW_WINDOW *canvas;
		canvas = GetActiveDrawWindow(sweeper->application);
		if(canvas != NULL)
		{
			set_pattern = cairo_pattern_create_for_surface(canvas->mixed_layer->surface_p);
			cairo_matrix_init_scale(&pattern_matrix, (FLOAT_T)canvas->width / allocation.width,
				(FLOAT_T)canvas->height / allocation.height);
			cairo_pattern_set_matrix(set_pattern, &pattern_matrix);
			cairo_set_source(cairo_p, set_pattern);
			cairo_paint(cairo_p);
			cairo_pattern_destroy(set_pattern);
		}
	}

	x_step = (FLOAT_T)allocation.width / sweeper->width;
	y_step = (FLOAT_T)allocation.height / sweeper->height;

	tile_scale_x = (FLOAT_T)sweeper->tile_width / x_step;
	tile_scale_y = (FLOAT_T)sweeper->tile_height / y_step;

	x_step = - x_step;
	y_step = - y_step;

	open_pattern = cairo_pattern_create_for_surface(sweeper->open_tile_surface);
	close_pattern = cairo_pattern_create_for_surface(sweeper->close_tile_surface);
	mine_pattern = cairo_pattern_create_for_surface(sweeper->mine_surface);
	flag_pattern = cairo_pattern_create_for_surface(sweeper->flag_surface);

	for(y=0; y<sweeper->height; y++)
	{
		cairo_matrix_init_scale(&pattern_matrix, tile_scale_x, tile_scale_y);
		cairo_matrix_translate(&pattern_matrix, 0, y * y_step);
		for(x=0; x<sweeper->width; x++)
		{
			opened = FLAG_CHECK(sweeper->opened, y*sweeper->width + x);
			if(opened)
			{
				set_pattern = open_pattern;
			}
			else
			{
				set_pattern = close_pattern;
			}
			cairo_pattern_set_matrix(set_pattern, &pattern_matrix);
			cairo_set_source(cairo_p, set_pattern);
			cairo_paint(cairo_p);

			set_pattern = NULL;
			if(opened && FLAG_CHECK(sweeper->mine, y*sweeper->width + x))
			{
				set_pattern = mine_pattern;
			}
			else if(! opened && FLAG_CHECK(sweeper->flaged, y*sweeper->width + x))
			{
				set_pattern = flag_pattern;
			}

			if(set_pattern != NULL)
			{
				cairo_pattern_set_matrix(set_pattern, &pattern_matrix);
				cairo_set_source(cairo_p, set_pattern);
				cairo_paint(cairo_p);
			}

			cairo_matrix_translate(&pattern_matrix, x_step, 0);
		}
	}

	cairo_pattern_destroy(open_pattern);
	cairo_pattern_destroy(close_pattern);
	cairo_pattern_destroy(mine_pattern);
	cairo_pattern_destroy(flag_pattern);

	x_step = - x_step;
	y_step = - y_step;

	display_y = 0;
	cairo_set_operator(cairo_p, CAIRO_OPERATOR_DIFFERENCE);
	cairo_set_source_rgb(cairo_p, 1, 1, 1);
	font_scale = (1 / MINIMUM(tile_scale_x, tile_scale_y)) * 30;
	cairo_set_font_size(cairo_p, font_scale);
	for(y=0; y<sweeper->height; y++, display_y += y_step)
	{
		display_x = 0;
		for(x=0; x<sweeper->width; x++, display_x += x_step)
		{
			if(FLAG_CHECK(sweeper->opened, y*sweeper->width + x)
				&& sweeper->num_mine[y][x] > 0
				&& !(FLAG_CHECK(sweeper->mine, y*sweeper->width + x)))
			{
				str[0] = '0' + sweeper->num_mine[y][x];
				cairo_text_extents(cairo_p, str, &character_info);
				cairo_move_to(cairo_p, display_x + (x_step - character_info.width * 1) * 0.5,
					display_y + (y_step + character_info.height * 1) * 0.5);
				cairo_show_text(cairo_p, str);
			}
		}
	}

#if GTK_MAJOR_VERSION <= 2
	cairo_destroy(cairo_p);
#endif

	return TRUE;
}

static gboolean ButtonPressCallback(GtkWidget* widget, GdkEventButton* event_info, MINE_SWEEPER* sweeper)
{
	GtkAllocation allocation;
	FLOAT_T tile_width, tile_height;
	int tile_x, tile_y;
	int result;

#if GTK_MAJOR_VERSION <= 2
	allocation = widget->allocation;
#else
	gtk_widget_get_allocation(widget, &allocation);
#endif

	tile_width = (FLOAT_T)allocation.width / sweeper->width;
	tile_height = (FLOAT_T)allocation.height / sweeper->height;

	tile_x = (int)(event_info->x / tile_width);
	tile_y = (int)(event_info->y / tile_height);

	if((sweeper->flags & (MINE_SWEEPER_FLAG_CLEARED | MINE_SWEEPER_FLAG_GAME_OVER)) != 0)
	{
		return TRUE;
	}

	if(event_info->button == 1)
	{
		if((sweeper->flags & MINE_SWEEPER_FLAG_STARTED) == 0)
		{
			InitializeBoard(sweeper, tile_x, tile_y);
			sweeper->flags |= MINE_SWEEPER_FLAG_STARTED;
		}
		result = OpenTile(sweeper, tile_x, tile_y);
		if(result < 0)
		{
			sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
			OpenAllTile(sweeper);
			gtk_widget_queue_draw(widget);
			return TRUE;
		}
		sweeper->num_opened += result;

		if(CheckSweeped(sweeper) != FALSE)
		{
			sweeper->flags |= MINE_SWEEPER_FLAG_CLEARED;
			OpenAllTile(sweeper);
			gtk_widget_queue_draw(widget);
			return TRUE;
		}

		if((event_info->state & GDK_BUTTON3_MASK) != 0)
		{
			sweeper->flags |= MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED;
		}
	}
	else if(event_info->button == 3)
	{
		if(tile_x >= 0 && tile_x < sweeper->width && tile_y >= 0 && tile_y < sweeper->height)
		{
			if(FLAG_CHECK(sweeper->flaged, tile_y*sweeper->width + tile_x))
			{
				FLAG_OFF(sweeper->flaged, tile_y*sweeper->width + tile_x);
			}
			else
			{
				FLAG_ON(sweeper->flaged, tile_y*sweeper->width + tile_x);
			}
		}

		if((event_info->state & GDK_BUTTON1_MASK) != 0)
		{
			sweeper->flags |= MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED;
		}
	}

	gtk_widget_queue_draw(widget);

	return TRUE;
}

static gboolean ButtonReleaseCallback(GtkWidget* widget, GdkEventButton* event_info, MINE_SWEEPER* sweeper)
{
	GtkAllocation allocation;
	FLOAT_T tile_width, tile_height;
	int tile_x, tile_y;
	int result;

#if GTK_MAJOR_VERSION <= 2
	allocation = widget->allocation;
#else
	gtk_widget_get_allocation(widget, &allocation);
#endif

	if((sweeper->flags & (MINE_SWEEPER_FLAG_CLEARED | MINE_SWEEPER_FLAG_GAME_OVER)) != 0)
	{
		return TRUE;
	}

	tile_width = (FLOAT_T)allocation.width / sweeper->width;
	tile_height = (FLOAT_T)allocation.height / sweeper->height;

	tile_x = (int)(event_info->x / tile_width);
	tile_y = (int)(event_info->y / tile_height);

	if(event_info->button == 1)
	{
		if((sweeper->flags & MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED) != 0)
		{
			if(tile_x >= 0 && tile_x < sweeper->width && tile_y >= 0 && tile_y < sweeper->height)
			{
				if(FLAG_CHECK(sweeper->opened, tile_y*sweeper->width + tile_x)
					&& CountAroundFlags(sweeper, tile_x, tile_y) == sweeper->num_mine[tile_y][tile_x])
				{
					result = OpenTile(sweeper, tile_x-1, tile_y-1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x, tile_y-1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x+1, tile_y-1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x-1, tile_y);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x+1, tile_y);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x-1, tile_y+1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x, tile_y+1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x+1, tile_y+1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;
				}
			}
		}

		sweeper->flags &= ~(MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED);
	}
	else if(event_info->button == 3)
	{
		if((sweeper->flags & MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED) != 0)
		{
			if(tile_x >= 0 && tile_x < sweeper->width && tile_y >= 0 && tile_y < sweeper->height)
			{
				if(FLAG_CHECK(sweeper->opened, tile_y*sweeper->width + tile_x)
					&& CountAroundFlags(sweeper, tile_x, tile_y) == sweeper->num_mine[tile_y][tile_x])
				{
					result = OpenTile(sweeper, tile_x-1, tile_y-1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x, tile_y-1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x+1, tile_y-1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x-1, tile_y);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x+1, tile_y);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x-1, tile_y+1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x, tile_y+1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;

					result = OpenTile(sweeper, tile_x+1, tile_y+1);
					if(result < 0)
					{
						sweeper->flags |= MINE_SWEEPER_FLAG_GAME_OVER;
						OpenAllTile(sweeper);
						gtk_widget_queue_draw(widget);
						return TRUE;
					}
					sweeper->num_opened += result;
				}
			}
		}
		
		sweeper->flags &= ~(MINE_SWEEPER_FLAG_LEFT_RIGHT_CLICKED);
	}

	return TRUE;
}

static void OnDestroyWindow(MINE_SWEEPER* sweeper)
{
	ReleaseMineSweeper(sweeper);
	//(void)g_module_close(sweeper->module);
	free(sweeper);
}

static void ChangeDifficulty(GtkWidget* menu_item, MINE_SWEEPER* sweeper)
{
	if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item)) != FALSE)
	{
		eMINE_SWEEPER_DIFFICULTY difficulty = (eMINE_SWEEPER_DIFFICULTY)g_object_get_data(
			G_OBJECT(menu_item), "difficulty");
		if(sweeper->difficulty != difficulty)
		{
			sweeper->difficulty = difficulty;
			ReleaseBuffer(sweeper);
			InitializeBuffer(sweeper);
			InitializeBoard(sweeper, 0, 0);
			SetClearLabel(sweeper);
			gtk_widget_queue_draw(sweeper->canvas);
		}
	}
}

static void ResetButtonClicked(MINE_SWEEPER* sweeper)
{
	ReleaseBuffer(sweeper);
	InitializeBuffer(sweeper);
	InitializeBoard(sweeper, 0, 0);
	SetClearLabel(sweeper);
	gtk_widget_queue_draw(sweeper->canvas);
}

G_MODULE_EXPORT int Execute(APPLICATION* app, const char* plug_in_name, GModule* module)
{
	MINE_SWEEPER *sweeper;
	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *canvas;
	GtkWidget *menu_bar;
	GtkWidget *menu;
	GtkWidget *sub_menu;
	GtkWidget *menu_item;
	GtkWidget *control[2];
	const char str[] = "<span font_desc=\"24\"></span>";

	sweeper = (MINE_SWEEPER*)malloc(sizeof(*sweeper));
	InitializeMineSweeper(sweeper);
	sweeper->application = app;
	sweeper->module = module;

	sweeper->window = window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	(void)g_signal_connect_swapped(G_OBJECT(window), "destroy",
		G_CALLBACK(OnDestroyWindow), sweeper);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(window), vbox);

	menu_bar = gtk_menu_bar_new();
	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);

	menu_item = gtk_menu_item_new_with_label(app->labels->preference.title);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);

	menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);

	menu_item = gtk_menu_item_new_with_label("Level");
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);

	sub_menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), sub_menu);

	control[0] = gtk_radio_menu_item_new_with_label(NULL, "Easy");
	g_object_set_data(G_OBJECT(control[0]), "difficulty", GINT_TO_POINTER(MINE_SWEEPER_DIFFICULTY_EASY));
	(void)g_signal_connect(G_OBJECT(control[0]), "toggled",
		G_CALLBACK(ChangeDifficulty), sweeper);
	gtk_menu_shell_append(GTK_MENU_SHELL(sub_menu), control[0]);

	control[1] = gtk_radio_menu_item_new_with_label(
		gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(control[0])), "Midium");
	g_object_set_data(G_OBJECT(control[1]), "difficulty", GINT_TO_POINTER(MINE_SWEEPER_DIFFICULTY_MIDIUM));
	(void)g_signal_connect(G_OBJECT(control[1]), "toggled",
		G_CALLBACK(ChangeDifficulty), sweeper);
	gtk_menu_shell_append(GTK_MENU_SHELL(sub_menu), control[1]);

	control[1] = gtk_radio_menu_item_new_with_label(
		gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(control[0])), "Hard");
	g_object_set_data(G_OBJECT(control[1]), "difficulty", GINT_TO_POINTER(MINE_SWEEPER_DIFFICULTY_HARD));
	(void)g_signal_connect(G_OBJECT(control[1]), "toggled",
		G_CALLBACK(ChangeDifficulty), sweeper);
	gtk_menu_shell_append(GTK_MENU_SHELL(sub_menu), control[1]);

	control[1] = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), control[1], FALSE, FALSE, 0);

	sweeper->clear_label = control[0] = gtk_label_new("");
	gtk_label_set_markup(GTK_LABEL(control[0]), str);
	gtk_box_pack_start(GTK_BOX(control[1]), control[0], FALSE, FALSE, 0);

	control[0] = gtk_button_new_with_label("Reset");
	gtk_box_pack_start(GTK_BOX(control[1]), control[0], FALSE, FALSE, 0);
	(void)g_signal_connect_swapped(G_OBJECT(control[0]), "clicked",
		G_CALLBACK(ResetButtonClicked), sweeper);

	sweeper->canvas = canvas = gtk_drawing_area_new();
	gtk_widget_set_size_request(canvas, 350, 350);
	gtk_widget_set_events(canvas, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
#if GTK_MAJOR_VERSION <= 2
	(void)g_signal_connect(G_OBJECT(canvas), "expose_event",
		G_CALLBACK(DisplayBoard), sweeper);
#else
	(void)g_signal_connect(G_OBJECT(canvas), "draw",
		G_CALLBACK(DisplayBoard), &sweeper);
#endif
	(void)g_signal_connect(G_OBJECT(canvas), "button_press_event",
		G_CALLBACK(ButtonPressCallback), sweeper);
	(void)g_signal_connect(G_OBJECT(canvas), "button_release_event",
		G_CALLBACK(ButtonReleaseCallback), sweeper);
	gtk_box_pack_start(GTK_BOX(vbox), canvas, TRUE, TRUE, 0);

	gtk_widget_show_all(window);

	return 1;
}

#if defined(_MSC_VER)

#ifdef _UNICODE
# if defined _M_IX86
#  pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='x86' publicKeyToken='6595b64144ccf1df' language='*'\"")
# elif defined _M_IA64
#  pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='ia64' publicKeyToken='6595b64144ccf1df' language='*'\"")
# elif defined _M_X64
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='amd64' publicKeyToken='6595b64144ccf1df' language='*'\"")
# else
#  pragma comment(linker,"/manifestdependency:\"type='win32' 
name='Microsoft.Windows.Common-Controls' version='6.0.0.0' 
processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
# endif
#endif

#pragma comment(lib, "KABURAGI.lib")

#if defined(USE_3D_LAYER) && USE_3D_LAYER != 0
# pragma comment(lib, "OpenGL32.lib")
# pragma comment(lib, "GlU32.lib")
# ifdef TEST //_DEBUG
#  pragma comment(lib, "tbb_debug.lib")
#  pragma comment(lib, "tbb_preview_debug.lib")
#  pragma comment(lib, "tbbmalloc_debug.lib")
#  pragma comment(lib, "tbbproxy_debug.lib")
#  pragma comment(lib, "tbbmalloc_proxy_debug.lib")
#  pragma comment(lib, "BulletCollision_vs2008_debug.lib")
#  pragma comment(lib, "BulletDynamics_vs2008_debug.lib")
#  pragma comment(lib, "BulletSoftBody_vs2008_debug.lib")
#  pragma comment(lib, "ConvexDecomposition_vs2008_debug.lib")
#  pragma comment(lib, "LinearMath_vs2008_debug.lib")
#  pragma comment(lib, "OpenGLSupport_vs2008_debug.lib")
# else
#  pragma comment(lib, "tbb.lib")
#  pragma comment(lib, "tbb_preview.lib")
#  pragma comment(lib, "tbbmalloc.lib")
#  pragma comment(lib, "tbbproxy.lib")
#  pragma comment(lib, "tbbmalloc_proxy.lib")
#  pragma comment(lib, "BulletCollision_vs2008.lib")
#  pragma comment(lib, "BulletDynamics_vs2008.lib")
#  pragma comment(lib, "BulletSoftBody_vs2008.lib")
#  pragma comment(lib, "ConvexDecomposition_vs2008.lib")
#  pragma comment(lib, "LinearMath_vs2008.lib")
#  pragma comment(lib, "OpenGLSupport_vs2008.lib")
# endif
#endif

# ifndef BUILD64BIT
#  if defined(USE_STATIC_LIB) && USE_STATIC_LIB != 0
#   ifdef _DEBUG
#    pragma comment(linker, "/NODEFAULTLIB:LIBCMTD.lib")
#    pragma comment(lib, "gtk_libsd.lib")
#    pragma comment(lib, "Dnsapi.lib")
#    pragma comment(lib, "ws2_32.lib")
#    pragma comment(lib, "Shlwapi.lib")
#    pragma comment(lib, "imm32.lib")
#    pragma comment(lib, "zlib.lib")
#    pragma comment(lib, "libpng.lib")
#    pragma comment(lib, "usp10.lib")
cairo_surface_t *
cairo_image_surface_create_from_png (const char	*filename)
{
	return 0;
}
/*
#    pragma comment(linker, "/NODEFAULTLIB:LIBCMTD.lib")
#    pragma comment(lib, "Dnsapi.lib")
#    pragma comment(lib, "ws2_32.lib")
#    pragma comment(lib, "pcre3d.lib")
#    pragma comment(lib, "Shlwapi.lib")
#    pragma comment(lib, "atk-1.0.lib")
#    pragma comment(lib, "cairo.lib")
#    pragma comment(lib, "expat.lib")
#    pragma comment(lib, "fontconfig.lib")
#    pragma comment(lib, "freetype.lib")
#    pragma comment(lib, "gailutil.lib")
#    pragma comment(lib, "gdk_pixbuf-2.0.lib")
#    pragma comment(lib, "gdk-win32-2.0.lib")
#    pragma comment(lib, "./STATIC_LIB/giod.lib")
#    pragma comment(lib, "./STATIC_LIB/glibd.lib")
#    pragma comment(lib, "./STATIC_LIB/gmoduled.lib")
#    pragma comment(lib, "./STATIC_LIB/gobjectd.lib")
#    pragma comment(lib, "./STATIC_LIB/gthreadd.lib")
#    pragma comment(lib, "gtk-win32-2.0.lib")
#    pragma comment(lib, "intl.lib")
#    pragma comment(lib, "libpng.lib")
#    pragma comment(lib, "pango-1.0.lib")
#    pragma comment(lib, "pangocairo-1.0.lib")
#    pragma comment(lib, "pangoft2-1.0.lib")
#    pragma comment(lib, "pangowin32-1.0.lib")
#    pragma comment(lib, "zlib.lib")
*/
#   else
#    pragma comment(lib, "./STATIC_LIB/libatk-1.0.a")
#    pragma comment(lib, "./STATIC_LIB/libcairo.a")
#    pragma comment(lib, "./STATIC_LIB/libcairo-gobject.a")
#    pragma comment(lib, "./STATIC_LIB/libcairo-script-interpreter.a")
#    pragma comment(lib, "./STATIC_LIB/libcharset.a")
#    pragma comment(lib, "./STATIC_LIB/libexpat.a")
#    pragma comment(lib, "./STATIC_LIB/libffi.a")
#    pragma comment(lib, "./STATIC_LIB/libfontconfig.a")
#    pragma comment(lib, "./STATIC_LIB/libfreetype.a")
#    pragma comment(lib, "./STATIC_LIB/libgailutil.a")
#    pragma comment(lib, "./STATIC_LIB/libgdk_pixbuf-2.0.a")
#    pragma comment(lib, "./STATIC_LIB/libgdk-win32-2.0.a")
#    pragma comment(lib, "./STATIC_LIB/libgio-2.0.a")
#    pragma comment(lib, "./STATIC_LIB/libglib-2.0.a")
#    pragma comment(lib, "./STATIC_LIB/libgmodule-2.0.a")
#    pragma comment(lib, "gobject-2.0.lib")
#    pragma comment(lib, "gthread-2.0.lib")
#    pragma comment(lib, "gtk-win32-2.0.lib")
#    pragma comment(lib, "intl.lib")
#    pragma comment(lib, "libpng.lib")
#    pragma comment(lib, "pango-1.0.lib")
#    pragma comment(lib, "pangocairo-1.0.lib")
#    pragma comment(lib, "pangoft2-1.0.lib")
#    pragma comment(lib, "pangowin32-1.0.lib")
#    pragma comment(lib, "zlib.lib")
#   endif
#  else
#  ifdef _DEBUG
#   pragma comment(linker, "/NODEFAULTLIB:LIBCMT")
#  else
#   if _MSC_VER >= 1600
#    pragma comment(linker, "/NODEFAULTLIB:LIBCMT")
#   endif
#  endif
#   if GTK_MAJOR_VERSION <= 2
#    pragma comment(lib, "atk-1.0.lib")
#    pragma comment(lib, "cairo.lib")
#    pragma comment(lib, "expat.lib")
#    pragma comment(lib, "fontconfig.lib")
#    pragma comment(lib, "freetype.lib")
#    pragma comment(lib, "gailutil.lib")
#    pragma comment(lib, "gdk_pixbuf-2.0.lib")
#    pragma comment(lib, "gdk-win32-2.0.lib")
#    pragma comment(lib, "gio-2.0.lib")
#    pragma comment(lib, "glib-2.0.lib")
#    pragma comment(lib, "gmodule-2.0.lib")
#    pragma comment(lib, "gobject-2.0.lib")
#    pragma comment(lib, "gthread-2.0.lib")
#    pragma comment(lib, "gtk-win32-2.0.lib")
#    pragma comment(lib, "intl.lib")
#    pragma comment(lib, "libpng.lib")
#    pragma comment(lib, "pango-1.0.lib")
#    pragma comment(lib, "pangocairo-1.0.lib")
#    pragma comment(lib, "pangoft2-1.0.lib")
#    pragma comment(lib, "pangowin32-1.0.lib")
#    if defined(_M_X64) || defined(_M_IA64)
#     pragma comment(lib, "zdll.lib")
#    else
#     pragma comment(lib, "zlib.lib")
#    endif
#   else
#    ifdef _DEBUG
//#     pragma comment(linker, "/NODEFAULTLIB:MSVCRTD")
#    else
#     pragma comment(linker, "/NODEFAULTLIB:MSVCRT")
#    endif

#    if defined(_M_X64) || defined(_M_IA64)
#     pragma comment(lib, "libpng.lib")
#    else
#     pragma comment(lib, "libpng.lib")
#    endif
#    pragma comment(lib, "zlib.lib")
#    pragma comment(lib, "cairo.lib")
#    pragma comment(lib, "atk-1.0.lib")
#    pragma comment(lib, "fontconfig.lib")
#    pragma comment(lib, "gailutil.lib")
#    pragma comment(lib, "gdk_pixbuf-2.0.lib")
#    pragma comment(lib, "gdk-win32-3.0.lib")
#    pragma comment(lib, "gio-2.0.lib")
#    pragma comment(lib, "glib-2.0.lib")
#    pragma comment(lib, "gmodule-2.0.lib")
#    pragma comment(lib, "gobject-2.0.lib")
#    pragma comment(lib, "gthread-2.0.lib")
#    pragma comment(lib, "gtk-win32-3.0.lib")
#    pragma comment(lib, "pango-1.0.lib")
#    pragma comment(lib, "pangocairo-1.0.lib")
#    pragma comment(lib, "pangoft2-1.0.lib")
#    pragma comment(lib, "pangowin32-1.0.lib")
#    pragma comment(lib, "libgettextsrc-0-18-2.lib")
#    pragma comment(lib, "libgettextpo-0.lib")
#    pragma comment(lib, "libgettextlib-0-18-2.lib")
#    pragma comment(lib, "libintl-8.lib")

#    pragma comment(lib, "libatk-1.0-0.lib")
#    pragma comment(lib, "libcairo-2.lib")
#    pragma comment(lib, "libcairo-gobject-2.lib")
#    pragma comment(lib, "libcairo-script-interpreter-2.lib")
#    pragma comment(lib, "libcroco-0.6-3.lib")
#    pragma comment(lib, "libffi-6.lib")
#    pragma comment(lib, "libfontconfig-1.lib")
#    pragma comment(lib, "libfreetype-6.lib")
#    pragma comment(lib, "libgailutil-3-0.lib")
#    pragma comment(lib, "libgdk_pixbuf-2.0-0.lib")
#    pragma comment(lib, "libgdk-3-0.lib")
#    pragma comment(lib, "libgio-2.0-0.lib")
#    pragma comment(lib, "libglib-2.0-0.lib")
#    pragma comment(lib, "libgmodule-2.0-0.lib")
#    pragma comment(lib, "libgthread-2.0-0.lib")
#    pragma comment(lib, "libgtk-3-0.lib")
#    pragma comment(lib, "libjasper-1.lib")
#    pragma comment(lib, "libjpeg-9.lib")
#    pragma comment(lib, "liblzma-5.lib")
#    pragma comment(lib, "libpango-1.0-0.lib")
#    pragma comment(lib, "libpangocairo-1.0-0.lib")
#    pragma comment(lib, "libpangoft2-1.0-0.lib")
#    pragma comment(lib, "libpangowin32-1.0-0.lib")
#    pragma comment(lib, "libpixman-1-0.lib")
#    pragma comment(lib, "librsvg-2-2.lib")
#    pragma comment(lib, "libtiff-5.lib")
#    pragma comment(lib, "libxml2-2.lib")
#    pragma comment(lib, "zdll.lib")

#   endif
#  endif
# else
#  ifdef _DEBUG
#   pragma comment(linker, "/NODEFAULTLIB:LIBCMTD")
#  else
//#   pragma comment(linker, "/NODEFAULTLIB:LIBCMT")
#  endif
#  if GTK_MAJOR_VERSION <= 2
#   pragma comment(lib, "atk-1.0.lib")
#   pragma comment(lib, "cairo.lib")
#   pragma comment(lib, "expat.lib")
#   pragma comment(lib, "fontconfig.lib")
#   pragma comment(lib, "freetype.lib")
#   pragma comment(lib, "gailutil.lib")
#   pragma comment(lib, "gdk_pixbuf-2.0.lib")
#   pragma comment(lib, "gdk-win32-2.0.lib")
#   pragma comment(lib, "gio-2.0.lib")
#   pragma comment(lib, "glib-2.0.lib")
#   pragma comment(lib, "gmodule-2.0.lib")
#   pragma comment(lib, "gobject-2.0.lib")
#   pragma comment(lib, "gthread-2.0.lib")
#   pragma comment(lib, "gtk-win32-2.0.lib")
#   pragma comment(lib, "intl.lib")
#   pragma comment(lib, "libpng.lib")
#   pragma comment(lib, "pango-1.0.lib")
#   pragma comment(lib, "pangocairo-1.0.lib")
#   pragma comment(lib, "pangoft2-1.0.lib")
#   pragma comment(lib, "pangowin32-1.0.lib")
#   if defined(_M_X64) || defined(_M_IA64)
#    pragma comment(lib, "zdll.lib")
#   else
#    pragma comment(lib, "zlib.lib")
#   endif
#  else
#    pragma comment(lib, "libpng15-15.lib")
#    pragma comment(lib, "zlib.lib")
#    pragma comment(lib, "libz.dll.a")
#    pragma comment(lib, "cairo.lib")
#    pragma comment(lib, "atk-1.0.lib")
#    pragma comment(lib, "fontconfig.lib")
#    pragma comment(lib, "gailutil.lib")
#    pragma comment(lib, "gdk_pixbuf-2.0.lib")
#    pragma comment(lib, "gdk-win32-3.0.lib")
#    pragma comment(lib, "gio-2.0.lib")
#    pragma comment(lib, "glib-2.0.lib")
#    pragma comment(lib, "gmodule-2.0.lib")
#    pragma comment(lib, "gobject-2.0.lib")
#    pragma comment(lib, "gthread-2.0.lib")
#    pragma comment(lib, "gtk-win32-3.0.lib")
#    pragma comment(lib, "pango-1.0.lib")
#    pragma comment(lib, "pangocairo-1.0.lib")
#    pragma comment(lib, "pangoft2-1.0.lib")
#    pragma comment(lib, "pangowin32-1.0.lib")
#    pragma comment(lib, "intl.lib")

#    pragma comment(lib, "libatk-1.0-0.lib")
#    pragma comment(lib, "libcairo.dll.a")
#    pragma comment(lib, "libcairo-gobject.dll.a")
#    pragma comment(lib, "libcairo-script-interpreter.dll.a")
#    pragma comment(lib, "libcroco-0.6.dll.a")
#    pragma comment(lib, "libffi.dll.a")
#    pragma comment(lib, "libfontconfig.dll.a")
#    pragma comment(lib, "libfreetype.dll.a")
#    pragma comment(lib, "libgailutil-3.dll.a")
#    pragma comment(lib, "libgdk_pixbuf-2.0.dll.a")
#    pragma comment(lib, "libgdk-3.dll.a")
#    pragma comment(lib, "libgio-2.0.dll.a")
#    pragma comment(lib, "libglib-2.0.dll.a")
#    pragma comment(lib, "libgmodule-2.0.dll.a")
#    pragma comment(lib, "libgthread-2.0.dll.a")
#    pragma comment(lib, "libgtk-3.dll.a")
#    pragma comment(lib, "libjasper.dll.a")
#    pragma comment(lib, "libjpeg.dll.a")
#    pragma comment(lib, "liblzma.dll.a")
#    pragma comment(lib, "libpango-1.0.dll.a")
#    pragma comment(lib, "libpangocairo-1.0.dll.a")
#    pragma comment(lib, "libpangoft2-1.0.dll.a")
#    pragma comment(lib, "libpangowin32-1.0.dll.a")
#    pragma comment(lib, "libpixman-1.dll.a")
#    pragma comment(lib, "librsvg-2.dll.a")
#    pragma comment(lib, "libtiff.dll.a")
#    pragma comment(lib, "libtiffxx.dll.a")
#    pragma comment(lib, "libxml2.dll.a")
#  endif
# endif
#endif
