#ifndef _INCLUDED_DRAW_WINDOW_H_
#define _INCLUDED_DRAW_WINDOW_H_

#include "configure.h"
#if defined(USE_3D_LAYER) && USE_3D_LAYER != 0
# include <GL/glew.h>
#endif
#include <gtk/gtk.h>
#include "lcms/lcms2.h"
#include "layer.h"
#include "history.h"
#include "selection_area.h"
#include "memory_stream.h"
#include "types.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum _eDRAW_WINDOW_FLAGS
{
	DRAW_WINDOW_HAS_SELECTION_AREA = 0x01,
	DRAW_WINDOW_UPDATE_ACTIVE_UNDER = 0x02,
	DRAW_WINDOW_UPDATE_ACTIVE_OVER = 0x04,
	DRAW_WINDOW_DISPLAY_HORIZON_REVERSE = 0x08,
	DRAW_WINDOW_EDIT_SELECTION = 0x10,
	DRAW_WINDOW_UPDATE_PART = 0x20,
	DRAW_WINDOW_DRAWING_STRAIGHT = 0x40,
	DRAW_WINDOW_SECOND_BG = 0x80,
	DRAW_WINDOW_TOOL_CHANGING = 0x100,
	DRAW_WINDOW_EDITTING_3D_MODEL = 0x200,
	DRAW_WINDOW_IS_FOCAL_WINDOW = 0x400,
	DRAW_WINDOW_INITIALIZED = 0x800
} eDRAW_WINDOW_FLAGS;

typedef struct _UPDATE_RECTANGLE
{
	FLOAT_T x, y;
	FLOAT_T width, height;
	cairo_t *cairo_p;
	cairo_surface_t *surface_p;
} UPDATE_RECTANGLE;

typedef struct _CALLBACK_IDS
{
	unsigned int display;
	unsigned int mouse_button_press;
	unsigned int mouse_move;
	unsigned int mouse_button_release;
	unsigned int mouse_wheel;
	unsigned int configure;
	unsigned int enter;
	unsigned int leave;
} CALLBACK_IDS;

typedef struct _DRAW_WINDOW
{
	uint8 channel;		// `l
	uint8 color_mode;	// J[[h(݂RGBA32̂)

	int16 degree;		// ]p(x@)

	gchar* file_name;	// t@C

	gchar* file_path;	// t@CpX

	// Ԃōĕ`ĂԊ֐ID
	guint timer_id;
	guint auto_save_id;

	// ^u؂ւÕC[r[̈ʒu
	int layer_view_position;

	// 摜̃IWiA
	int original_width, original_height;
	// `̈̕A(4̔{)
	int width, height;
	// s̃oCgAC[ꖇ̃oCg
	int stride, pixel_buf_size;
	// ]pɃ}[WƂ`̈̍LAs̃oCg
	int disp_size, disp_stride;
	// ]̍WvZp
	FLOAT_T half_size;
	FLOAT_T angle;		// ]p(ʓx@)
	// ]p̎Op֐̒l
	FLOAT_T sin_value, cos_value;
	// ]ł̈ړ
	FLOAT_T trans_x, trans_y;
	// J[\W
	FLOAT_T cursor_x, cursor_y;
	FLOAT_T before_cursor_x, before_cursor_y;
	// O`FbÑJ[\W
	FLOAT_T before_x, before_y;
	// ŌɃNbN܂̓hbOꂽW
	FLOAT_T last_x, last_y;
	// ŌɃNbN܂̓hbOꂽ̕M
	FLOAT_T last_pressure;
	// J[\W␳p
	FLOAT_T add_cursor_x, add_cursor_y;
	// ϖ@u␳ł̍Wϊp
	FLOAT_T rev_add_cursor_x, rev_add_cursor_y;
	// \p̃p^[
	cairo_pattern_t *rotate;
	// ʕXVp
	UPDATE_RECTANGLE update, temp_update;
	// `̈XN[̍W
	int scroll_x, scroll_y;
	// ʍXṼNbsOp
	int update_clip_area[4][2];

	// WindowsLinuxŃCxǧĂ΂鏇قȂ悤Ȃ̂ŉp
	GdkModifierType state;

	// R[obN֐IDL
	CALLBACK_IDS callbacks;

	uint8 *back_ground;		// wĩsNZf[^
	uint8 *brush_buffer;	// uVp̃obt@

	LAYER* layer;			// ԉ̃C[
	// \pAGtFNgpAuVJ[\\p̈ꎞۑ
	LAYER* disp_layer, *effect, *disp_temp, *scaled_mixed;
	// ANeBuȃC[&C[Zbgւ̃|C^
		// yѕ\C[C[
	LAYER* active_layer, *active_layer_set, *mixed_layer;
	// ƗpAꎞۑpAÍ͈AANeBuC[艺̃C[
	LAYER *work_layer, *temp_layer,
		*selection, *under_active;
	// }XNƃ}XNKpÖꎞۑp
	LAYER* mask, *mask_temp;
	// eNX`p
	LAYER* texture;
	// \pp^[
	cairo_pattern_t *mixed_pattern;
	// `l݂̂̃}XNpT[tF[X
	cairo_surface_t *alpha_surface, *alpha_temp, *gray_mask_temp;
	// `l݂̂̃C[W
	cairo_t *alpha_cairo, *alpha_temp_cairo, *gray_mask_cairo;

	uint16 num_layer;		// C[̐
	uint16 zoom;			// gEk
	FLOAT_T zoom_rate;		// _^̊gEk
	FLOAT_T rev_zoom;		// gEk̋t

	// 𑜓x
	int16 resolution;

	// I͈̓Aj[Vp̃^C}[
	GTimer *timer;

	// ۑp̃^C}[
	GTimer *auto_save_timer;

	// I͈͕\pf[^
	SELECTION_AREA selection_area;

	// f[^
	HISTORY history;

	// `̈̃EBWFbg
	GtkWidget* window;

	// `̈̃XN[
	GtkWidget* scroll;

	// \XṼtO
	unsigned int flags;

	// ό`p̃f[^
	struct _TRANSFORM_DATA* transform;

	// 2߂̔wiF
	uint8 second_back_ground[3];

	// \tB^[̏
	uint8 display_filter_mode;

	// ICCvt@C̃f[^
	char *icc_profile_path;
	void *icc_profile_data;
	int32 icc_profile_size;
	// ICCvt@C̓e
	cmsHPROFILE input_icc;
	// ICCvt@CɂFϊp
	cmsHTRANSFORM icc_transform;

	// ǏLoX
	struct _DRAW_WINDOW *focal_window;

	// AvP[VS̊Ǘp\̂ւ̃|C^
	struct _APPLICATION* app;

#if defined(USE_3D_LAYER) && USE_3D_LAYER != 0
	// 3DfOpf[^
	void *first_project;
	// OpenGLΉEBWFbg
	GtkWidget *gl_area;
#endif
} DRAW_WINDOW;

// ֐̃vg^Cv錾
/***************************************************************
* CreateDrawWindow֐                                         *
* `̈쐬                                           *
*                                                          *
* width		: LoX̕                                   *
* height	: LoX̍                                 *
* channel	: LoX̃`l(RGB:3, RGBA:4)          *
* name		: LoX̖O                                 *
* note_book	: `̈^uEBWFbg                         *
* window_id	: `̈z񒆂ID                               *
* app		: AvP[V̏Ǘ\̂̃AhX *
* Ԃl                                                       *
*	`̈̏Ǘ\̂̃AhX                   *
***************************************************************/
EXTERN DRAW_WINDOW* CreateDrawWindow(
	int32 width,
	int32 height,
	uint8 channel,
	const gchar* name,
	GtkWidget* note_book,
	uint16 window_id,
	struct _APPLICATION* app
);

/***************************************************************
* CreateTempDrawWindow֐                                     *
* ꎞIȕ`̈쐬                                   *
*                                                          *
* width		: LoX̕                                   *
* height	: LoX̍                                 *
* channel	: LoX̃`l(RGB:3, RGBA:4)          *
* name		: LoX̖O                                 *
* note_book	: `̈^uEBWFbg                         *
* window_id	: `̈z񒆂ID                               *
* app		: AvP[V̏Ǘ\̂̃AhX *
* Ԃl                                                       *
*	`̈̏Ǘ\̂̃AhX                   *
***************************************************************/
EXTERN DRAW_WINDOW* CreateTempDrawWindow(
	int32 width,
	int32 height,
	uint8 channel,
	const gchar* name,
	GtkWidget* note_book,
	uint16 window_id,
	struct _APPLICATION* app
);

/***************************************
* DeleteDrawWindow֐                 *
* `̈̏폜                 *
*                                  *
* window	: `̈̏̃AhX *
***************************************/
EXTERN void DeleteDrawWindow(DRAW_WINDOW** window);

/*********************************************
* TimerCallBack֐                          *
* (1/60b)ŌĂяoR[obN֐ *
*                                        *
* window	: Ή`̈               *
* Ԃl                                     *
*	TRUE                                 *
*********************************************/
EXTERN gboolean TimerCallBack(DRAW_WINDOW* window);

/*******************************************************
* SetDrawWindowCallbacks֐                           *
* `̈̃R[obN֐̐ݒs               *
*                                                  *
* widget	: R[obN֐ZbgEBWFbg *
* window	: `̈̏                           *
*******************************************************/
EXTERN void SetDrawWindowCallbacks(
	GtkWidget* widget,
	DRAW_WINDOW* window
);

/***************************************************
* DisconnectDrawWindowCallbacks֐                *
* `̈̃R[obN֐~߂               *
*                                              *
* widget	: R[obN֐~߂EBWFbg *
* window	: `̈̏                       *
***************************************************/
EXTERN void DisconnectDrawWindowCallbacks(
	GtkWidget* widget,
	DRAW_WINDOW* window
);

/*****************************************************
* SwapDrawWindowFromMemoryStream֐                 *
* [̕`̈f[^Ɠւ             *
*                                                *
* window	: `̈̏                         *
* stream	: [̕`̈̃f[^Xg[ *
*****************************************************/
EXTERN void SwapDrawWindowFromMemoryStream(DRAW_WINDOW* window, MEMORY_STREAM_PTR stream);

/***********************************************
* OnCloseDrawWindow֐                        *
* ^uƂ̃R[obN֐       *
*                                          *
* data	: `̈̃f[^                     *
* page	: ^uID                       *
* Ԃl                                       *
*	鑀̒~:TRUE 鑀쑱s:FALSE *
***********************************************/
EXTERN gboolean OnCloseDrawWindow(void* data, gint page);

/***********************************************************
* GetWindowID֐                                          *
* `̈ID擾                                   *
*                                                      *
* window	: `̈̏                               *
* app		: 	AvP[VǗ\̂̃AhX *
* Ԃl                                                   *
*	`̈ID (sȕ`̈Ȃ-1)                  *
***********************************************************/
EXTERN int GetWindowID(DRAW_WINDOW* window, struct _APPLICATION* app);

/*********************************
* DrawWindowChangeZoom֐       *
* `̈̊gkύX *
*                            *
* window	: `̈̏     *
* zoom		: Vgk   *
*********************************/
EXTERN void DrawWindowChangeZoom(
	DRAW_WINDOW* window,
	int16 zoom
);

/*****************************************
* FlipDrawWindowHorizontally֐         *
* `̈𐅕]                 *
*                                    *
* window	: ]`̈̏ *
*****************************************/
EXTERN void FlipDrawWindowHorizontally(DRAW_WINDOW* window);

/*****************************************
* FlipDrawWindowVertically֐           *
* `̈𐂒]                 *
*                                    *
* window	: ]`̈̏ *
*****************************************/
EXTERN void FlipDrawWindowVertically(DRAW_WINDOW* window);

/***************************************
* LayerAlpha2SelectionArea֐         *
* C[̕sI͈͂ɂ *
*                                  *
* window	: `̈̏           *
***************************************/
EXTERN void LayerAlpha2SelectionArea(DRAW_WINDOW* window);

/*****************************************
* LayerAlphaAddSelectionArea֐         *
* C[̕sI͈͂ɉ *
*                                    *
* window	: `̈̏             *
*****************************************/
EXTERN void LayerAlphaAddSelectionArea(DRAW_WINDOW* window);

/*****************************
* MergeAllLayer֐          *
* SẴC[   *
*                        *
* window	: `̈̏ *
*****************************/
EXTERN void MergeAllLayer(DRAW_WINDOW* window);

/*******************************************
* ChangeDrawWindowResolution֐           *
* 𑜓xύX                         *
*                                      *
* window		: 𑜓xύX`̈ *
* new_width		: V                 *
* new_height	: V               *
*******************************************/
EXTERN void ChangeDrawWindowResolution(DRAW_WINDOW* window, int32 new_width, int32 new_height);

/*************************************************
* AddChangeDrawWindowResolutionHistory֐       *
* 𑜓xύX̗f[^ǉ               *
*                                            *
* window		: 𑜓xύX`̈̏ *
* new_width		: V                       *
* new_height	: V                     *
*************************************************/
EXTERN void AddChangeDrawWindowResolutionHistory(
	DRAW_WINDOW* window,
	int32 new_width,
	int32 new_height
);

/*******************************************
* ChangeDrawWindowSize֐                 *
* `̈̃TCYύX               *
*                                      *
* window		: 𑜓xύX`̈ *
* new_width		: V                 *
* new_height	: V               *
*******************************************/
EXTERN void ChangeDrawWindowSize(DRAW_WINDOW* window, int32 new_width, int32 new_height);

/*****************************************
* UpdateDrawWindowClippingArea           *
* ʍXVɃNbsÖXV *
*                                    *
* window	: `̈̏             *
*****************************************/
EXTERN void UpdateDrawWindowClippingArea(DRAW_WINDOW* window);

/*************************************************
* ClipUpdateArea֐                             *
* ʂ̃XN[ɓĂ镔ŃNbsO *
*                                            *
* window	: `̈̏                     *
* cairo_p	: Cairo                          *
*************************************************/
EXTERN void ClipUpdateArea(DRAW_WINDOW* window, cairo_t* cairo_p);

/*******************************************************************
* DrawWindowSetIccProfile֐                                      *
* LoXICCvt@C蓖Ă                          *
*                                                              *
* window	: `̈̏(icc_profile_dataɃf[^蓖čς) *
* data_size	: ICCvt@C̃f[^̃oCg                    *
* ask_set	: \tgv[t\Kp邩q˂邩ۂ         *
*******************************************************************/
EXTERN void DrawWindowSetIccProfile(DRAW_WINDOW* window, int32 data_size, gboolean ask_set);

/*************************************************
* AddChangeDrawWindowSizeHistory֐             *
* LoXTCYύX̗f[^ǉ     *
*                                            *
* window		: 𑜓xύX`̈̏ *
* new_width		: V                       *
* new_height	: V                     *
*************************************************/
EXTERN void AddChangeDrawWindowSizeHistory(
	DRAW_WINDOW* window,
	int32 new_width,
	int32 new_height
);

/*************************************************************
* AddLayer֐                                               *
* `̈ɃC[ǉ                               *
*                                                        *
* canvas		: C[ǉ`̈                 *
* layer_type	: ǉ郌C[̃^Cv(ʏAxNg) *
* layer_name	: ǉ郌C[̖O                     *
* prev_layer	: ǉC[̉ɂ郌C[         *
* Ԃl                                                     *
*	:C[\̂̃AhX	s:NULL                *
*************************************************************/
EXTERN LAYER* AddLayer(
	DRAW_WINDOW* window,
	eLAYER_TYPE layer_type,
	const char* layer_name,
	LAYER* prev_layer
);

EXTERN void RasterizeVectorLine(
	DRAW_WINDOW* window,
	VECTOR_LAYER* layer,
	VECTOR_LINE* line,
	VECTOR_LAYER_RECTANGLE* rect
);

EXTERN void RasterizeVectorLayer(
	DRAW_WINDOW* window,
	LAYER* target,
	VECTOR_LAYER* layer
);

/***********************************************
* ReadVectorLineData֐                       *
* xNgC[̃f[^ǂݍ           *
*                                          *
* data		: xNgC[̃f[^(k) *
* target	: f[^ǂݍރC[         *
* Ԃl                                       *
*	ǂݍ񂾃oCg                         *
***********************************************/
EXTERN uint32 ReadVectorLineData(uint8* data, LAYER* target);

/*********************************************************
* WriteVectorLineData֐                                *
* xNgC[̃f[^o                     *
*                                                    *
* target			: f[^oC[           *
* dst				: õ|C^               *
* write_func		: oɎg֐|C^         *
* data_stream		: kÕf[^쐬Xg[ *
* write_stream		: of[^쐬Xg[ *
* compress_level	: ZIPk̃x                    *
*********************************************************/
EXTERN void WriteVectorLineData(
	LAYER* target,
	void* dst,
	stream_func_t write_func,
	MEMORY_STREAM* data_stream,
	MEMORY_STREAM* write_stream,
	int compress_level
);

/***********************************************
* IsVectorLineInSelectionArea֐              *
* I͈͓ۂ𔻒肷               *
*                                          *
* window			: `̈̏           *
* selection_pixels	: I͈͂̃sNZf[^ *
* line				: 肷               *
* Ԃl                                       *
*	I͈͓:1	I͈͊O:0               *
***********************************************/
EXTERN int IsVectorLineInSelectionArea(
	DRAW_WINDOW* window,
	uint8* selection_pixels,
	VECTOR_LINE* line
);

/*******************************************
* AddControlPointHistory֐               *
* _ǉ̗쐬               *
*                                      *
* window	: `̈̏               *
* layer		: _ǉC[     *
* line		: _ǉC       *
* point		: ǉ_̃AhX     *
* tool_name	: _ǉc[̖O *
*******************************************/
EXTERN void AddControlPointHistory(
	DRAW_WINDOW* window,
	LAYER* layer,
	VECTOR_LINE* line,
	VECTOR_POINT* point,
	const char* tool_name
);

/***************************************************
* AddTopLineControlPointHistory֐                *
* ܂c[Ȑc[ł̐_ǉǉ *
*                                              *
* window		: `̈̏                   *
* layer			: _ǉC[         *
* line			: _ǉC           *
* point			: ǉ_                   *
* tool_name		: _ǉc[̖O     *
* add_line_flag	: CǉƂ̃tO     *
***************************************************/
EXTERN void AddTopLineControlPointHistory(
	DRAW_WINDOW* window,
	LAYER* layer,
	VECTOR_LINE* line,
	VECTOR_POINT* point,
	const char* tool_name,
	uint8 add_line_flag
);

/***********************************************************
* AddDeleteLinesHistory֐                                *
* ̐𓯎ɍ폜ۂ̗f[^ǉ             *
*                                                      *
* window		: `̈̏Ǘ\̂̃AhX *
* active_layer	: ANeBuȃC[                     *
* line_data		: 폜̃f[^                       *
* line_indexes	: 폜̈ԉ琔CfbNX *
* num_lines		: 폜̐                           *
* tool_name		: 폜Ɏgpc[̖O               *
***********************************************************/
EXTERN void AddDeleteLinesHistory(
	DRAW_WINDOW* window,
	LAYER* active_layer,
	VECTOR_LINE* line_data,
	uint32* line_indexes,
	uint32 num_lines,
	const char* tool_name
);

/*******************************************
* DeleteLayerSet֐                       *
* C[Zbg̍폜s               *
*                                      *
* layer_set	: 폜郌C[Zbg       *
* window	: C[Zbg`̈ *
*******************************************/
EXTERN void DeleteLayerSet(LAYER* layer_set, DRAW_WINDOW* window);

/*************************************************
* MixLayerSet֐                                *
* C[Zbg                         *
*                                            *
* bottom	: C[Zbg̈ԉ̃C[   *
* next		: Ɏɍ郌C[       *
* window	: `̈Ǘ\̂̃AhX *
*************************************************/
EXTERN void MixLayerSet(LAYER* bottom, LAYER** next, DRAW_WINDOW* window);

/***************************************************************
* MixLayerSetActiveOver֐                                    *
* C[Zbg̃ANeBuC[ȏ̃C[ *
*                                                          *
* start		: ANeBuC[                               *
* next		: ̎ɍ郌C[                     *
* window	: `̈Ǘ\̂̃AhX               *
***************************************************************/
EXTERN void MixLayerSetActiveOver(LAYER* start, LAYER** next, DRAW_WINDOW* window);

EXTERN void RenderTextLayer(DRAW_WINDOW* window, struct _LAYER* target, TEXT_LAYER* layer);

EXTERN void DisplayTextLayerRange(DRAW_WINDOW* window, TEXT_LAYER* layer);

EXTERN void DisplayEditSelection(DRAW_WINDOW* window);

/***************************************
* g_part_layer_blend_funcsz         *
* C[֐ւ̃|C^z *
*                                  *
* src		: 郌C[         *
* update	: XV͈͂̏           *
***************************************/
extern void (*g_part_layer_blend_funcs[])(LAYER* src, UPDATE_RECTANGLE* update);

/*****************************************
* AutoSave֐                           *
* obNAbvƂăt@Cۑ   *
*                                    *
* window	: obNAbv`̈ *
*****************************************/
EXTERN void AutoSave(DRAW_WINDOW* window);

/***************************************************
* GetBlendedUnderLayer֐                         *
* Ώۂ艺̃C[C[擾 *
*                                              *
* target			: Ώۂ̃C[               *
* window			: `̈̏               *
* use_back_ground	: wiFgp邩ǂ     *
* Ԃl                                           *
*	C[                               *
***************************************************/
EXTERN LAYER* GetBlendedUnderLayer(LAYER* target, DRAW_WINDOW* window, int use_back_ground);

EXTERN void DivideLinesUndo(DRAW_WINDOW* window, void* p);
EXTERN void DivideLinesRedo(DRAW_WINDOW* window, void* p);

EXTERN void ScrollSizeChangeEvent(GtkWidget* scroll, GdkRectangle* size, DRAW_WINDOW* window);

EXTERN gboolean DrawWindowConfigurEvent(GtkWidget* widget, GdkEventConfigure* event_info, DRAW_WINDOW* window);

EXTERN void UpdateDrawWindow(DRAW_WINDOW* window);

/*********************************
* Change2FocalMode֐           *
* ǏLoX[hɈڍs *
*                            *
* parent_window	: eLoX   *
*********************************/
EXTERN void Change2FocalMode(DRAW_WINDOW* parent_window);

/*********************************
* ReturnFromFocalMode֐        *
* ǏLoX[h߂   *
*                            *
* parent_window	: eLoX   *
*********************************/
EXTERN void ReturnFromFocalMode(DRAW_WINDOW* parent_window);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef _INCLUDED_DRAW_WINDOW_H_
