#ifndef _INCLUDED_LAYER_H_
#define _INCLUDED_LAYER_H_

#define MAX_LAYER_NAME_LENGTH 256

#include <gtk/gtk.h>
#include <cairo.h>
#include "configure.h"
#include "types.h"
#include "vector_layer.h"
#include "text_layer.h"
#include "layer_set.h"
#include "pattern.h"
#include "texture.h"

#ifdef __cplusplus
extern "C" {
#endif

#define LAYER_CHAIN_BUFFER_SIZE 1024
#define MAX_LAYER_EXTRA_DATA_NUM 8

typedef enum _eLAYER_FLAGS
{
	LAYER_FLAG_INVISIBLE = 0x01,
	LAYER_MASKING_WITH_UNDER_LAYER = 0x02,
	LAYER_LOCK_OPACITY = 0x04,
	LAYER_CHAINED = 0x08,
	LAYER_SET_CLOSE = 0x10
} eLAYER_FLAGS;

typedef enum _eLAYER_TYPE
{
	TYPE_NORMAL_LAYER,
	TYPE_VECTOR_LAYER,
	TYPE_TEXT_LAYER,
	TYPE_LAYER_SET,
	TYPE_3D_LAYER,
	NUM_LAYER_TYPE
} eLAYER_TYPE;

typedef enum _eLAYER_BLEND_MODE
{
	LAYER_BLEND_NORMAL,
	LAYER_BLEND_ADD,
	LAYER_BLEND_MULTIPLY,
	LAYER_BLEND_SCREEN,
	LAYER_BLEND_OVERLAY,
	LAYER_BLEND_LIGHTEN,
	LAYER_BLEND_DARKEN,
	LAYER_BLEND_DODGE,
	LAYER_BLEND_BURN,
	LAYER_BLEND_HARD_LIGHT,
	LAYER_BLEND_SOFT_LIGHT,
	LAYER_BLEND_DIFFERENCE,
	LAYER_BLEND_EXCLUSION,
	LAYER_BLEND_HSL_HUE,
	LAYER_BLEND_HSL_SATURATION,
	LAYER_BLEND_HSL_COLOR,
	LAYER_BLEND_HSL_LUMINOSITY,
	LAYER_BLEND_BINALIZE,
	LAYER_BLEND_SLELECTABLE_NUM,
	LAYER_BLEND_COLOR_REVERSE = LAYER_BLEND_SLELECTABLE_NUM,
	LAYER_BLEND_GREATER,
	LAYER_BLEND_ALPHA_MINUS,
	LAYER_BLEND_SOURCE,
	LAYER_BLEND_ATOP,
	LAYER_BLEND_SOURCE_OVER,
	LAYER_BLEND_OVER
} eLAYER_BLEND_MODE;

/***************************
* EXTRA_LAYER_DATA\   *
* C[̒ǉi[ *
***************************/
typedef struct _EXTRA_LAYER_DATA
{
	char *name;
	size_t data_size;
	void *data;
} EXTRA_LAYER_DATA;

/***********************
* LAYER\          *
* C[̏i[ *
***********************/
typedef struct _LAYER
{
	uint8 *pixels;			// sNZf[^
	gchar* name;			// C[
	uint16 layer_type;		// C[^Cv(m[}AxNgAeLXg)
	uint16 layer_mode;		// C[̍[h
	int32 x, y;				// C[̍W
	// C[̕AAs̃oCg
	int width, height, stride;
	unsigned int flags;		// C[\E\̃tO
	int8 alpha;				// C[̕sx
	int8 channel;			// C[̃`l
	uint8 num_extra_data;	// ǉ̐

	// CAIRȌ݃T[tFCX
	cairo_surface_t *surface_p;
	cairo_t *cairo_p;	// CAIROReLXg

	// layer_datap
		// xNgC[AeLXgC[̏i[
	union
	{
		VECTOR_LAYER* vector_layer_p;
		TEXT_LAYER* text_layer_p;
		LAYER_SET* layer_set_p;

#if defined(USE_3D_LAYER) && USE_3D_LAYER != 0
		// 3DfOpf[^
		void *project;
#endif
	} layer_data;

	// ÕC[ւ̃|C^
	struct _LAYER *prev, *next;
	// C[Zbgւ̃|C^
	struct _LAYER *layer_set;
	// C[r[̃EBWFbg
	struct _LAYER_WIDGET *widget;

	// ǉ
	EXTRA_LAYER_DATA extra_data[MAX_LAYER_EXTRA_DATA_NUM];

	// t@Cǂݍݎ3Dff[^
#if defined(USE_3D_LAYER) && USE_3D_LAYER != 0
	void *modeling_data;
	size_t modeling_data_size;
#endif

	// `̈ւ̃|C^
	struct _DRAW_WINDOW *window;
} LAYER;

/*************************************
* LAYER_BASE_DATA\              *
* C[̊{BoɎgp *
*************************************/
typedef struct _LAYER_BASE_DATA
{
	uint16 layer_type;	// C[^Cv(m[}AxNgAeLXg)
	uint16 layer_mode;	// C[̍[h
	gint32 x, y;		// C[̍W
	// C[̕A
	gint32 width, height;
	guint32 flags;		// C[\E\̃tO
	int8 alpha;			// C[̕sx
	int8 channel;		// C[̃`l
	int8 layer_set;		// C[Zbg̃C[ۂ
} LAYER_BASE_DATA;

/********************
* RGAB_DATA\   *
* 1sNZ̃f[^ *
********************/
typedef struct _RGBA_DATA
{
	uint8 r, g, b, a;
} RGBA_DATA;

// ֐̃vg^Cv錾
EXTERN LAYER* CreateLayer(
	int32 x,
	int32 y,
	int32 width,
	int32 height,
	int8 channel,
	eLAYER_TYPE layer_type,
	LAYER* prev_layer,
	LAYER* next_layer,
	const char *name,
	struct _DRAW_WINDOW* window
);

/*****************************************************
* CreateDispTempLayer֐                            *
* \pꎞۑC[쐬                       *
*                                                *
* x				: C[XW                *
* y				: C[YW                *
* width			: C[̕                       *
* height		: C[̍                     *
* channel		: C[̃`l             *
* layer_type	: C[̃^Cv                   *
* window		: ǉ郌C[Ǘ`̈ *
* Ԃl                                             *
*	ꂽ\̂̃AhX                     *
*****************************************************/
LAYER* CreateDispTempLayer(
	int32 x,
	int32 y,
	int32 width,
	int32 height,
	int8 channel,
	eLAYER_TYPE layer_type,
	struct _DRAW_WINDOW* window
);

/*************************************
* g_layer_blend_funcsz            *
* C[p֐ւ̃|C^z *
*                                *
* src	: 㑤̃C[             *
* dst	: ̃C[             *
*************************************/
EXTERN void (*g_layer_blend_funcs[])(LAYER* src, LAYER* dst);

/*********************************************
* DeleteLayer֐                            *
* C[폜                         *
*                                        *
* layer	: 폜郌C[|C^̃AhX *
*********************************************/
EXTERN void DeleteLayer(LAYER** layer);

/*********************************************************
* RemoveLayer֐                                        *
* C[폜(LoXɓo^Ă̂̂) *
*                                                    *
* target	: 폜郌C[                           *
*********************************************************/
EXTERN void RemoveLayer(LAYER* target);

/*********************************************
* DeleteTempLayer֐                        *
* ꎞIɍ쐬C[폜         *
*                                        *
* layer	: 폜郌C[|C^̃AhX *
*********************************************/
EXTERN void DeleteTempLayer(LAYER** layer);

/*********************************
* AddDeleteLayerHistory֐      *
* C[̍폜f[^쐬 *
*                            *
* window	: `̈̃f[^   *
* target	: 폜郌C[   *
*********************************/
EXTERN void AddDeleteLayerHistory(
	struct _DRAW_WINDOW* window,
	LAYER* target
);

/***********************************************************
* ResizeLayerBuffer֐                                    *
* C[̃sNZf[^̃TCYύX               *
*                                                      *
* target		: sNZf[^̃TCYύX郌C[ *
* new_width		: VC[̕                       *
* new_height	: VC[̍                     *
***********************************************************/
EXTERN void ResizeLayerBuffer(
	LAYER* target,
	int32 new_width,
	int32 new_height
);

/*******************************************
* ResizeLayer֐                          *
* C[̃TCYύX(gkL)   *
*                                      *
* target		: TCYύX郌C[ *
* new_width		: VC[̕       *
* new_height	: VC[̍     *
*******************************************/
EXTERN void ResizeLayer(
	LAYER* target,
	int32 new_width,
	int32 new_height
);

/*******************************************
* ChangeLayerSize֐                      *
* C[̃TCYύX(gk)   *
*                                      *
* target		: TCYύX郌C[ *
* new_width		: VC[̕       *
* new_height	: VC[̍     *
*******************************************/
EXTERN void ChangeLayerSize(
	LAYER* target,
	int32 new_width,
	int32 new_height
);

EXTERN int CorrectLayerName(const LAYER* bottom_layer, const char* name);

EXTERN void ChangeActiveLayer(struct _DRAW_WINDOW* window, LAYER* layer);

/*******************************
* LayerMergeDown֐           *
* ̃C[ƌ       *
*                          *
* target	: 郌C[ *
*******************************/
EXTERN void LayerMergeDown(LAYER* target);

/*******************************************
* AddLayerMergeDownHistory֐             *
* ̃C[ƌ̗f[^ǉ *
*                                      *
* window	: `̈̏               *
* target	: 郌C[             *
*******************************************/
EXTERN void AddLayerMergeDownHistory(
	struct _DRAW_WINDOW* window,
	LAYER* target
);

EXTERN void ChangeLayerOrder(LAYER* change_layer, LAYER* new_prev, LAYER** bottom);

EXTERN LAYER* SearchLayer(LAYER* bottom_layer, const gchar* name);

EXTERN int32 GetLayerID(const LAYER* bottom, const LAYER* prev, uint16 num_layer);

/*********************************
* AddDeleteLayerHistory֐      *
* C[̍폜f[^쐬 *
*                            *
* window	: `̈̃f[^   *
* target	: 폜郌C[   *
*********************************/
EXTERN void AddNewLayerHistory(
	const LAYER* new_layer,
	uint16 layer_type
);

/*********************************************
* AddNewLayerWithImageHistory֐            *
* 摜f[^tł̃C[ǉ̗ǉ   *
*                                        *
* new_layer		: ǉC[           *
* pixels		: 摜f[^̃sNZf[^ *
* width			: 摜f[^̕             *
* height		: 摜f[^̍           *
* stride		: 摜f[^s̃oCg *
* channel		: 摜f[^̃`l   *
* history_name	: ̖O                 *
*********************************************/
EXTERN void AddNewLayerWithImageHistory(
	const LAYER* new_layer,
	uint8* pixels,
	int32 width,
	int32 height,
	int32 stride,
	uint8 channel,
	const char* history_name
);

/*************************************************
* AddChangeLayerOrderHistory֐                 *
* C[̏ύX̗f[^ǉ           *
*                                            *
* change_layer	: ύXC[           *
* before_prev	: ύXȎÕC[       *
* after_prev	: ύX̑ÕC[       *
* before_parent	: ύXȌC[Zbg *
*************************************************/
EXTERN void AddChangeLayerOrderHistory(
	const LAYER* change_layer,
	const LAYER* before_prev,
	const LAYER* after_prev,
	const LAYER* before_parent
);

/***********************************************************
* AddChangeLyaerSetHistory֐                             *
* C[̏C[ZbgύX̗f[^ǉ       *
*                                                      *
* change_layer	: C[ZbgύX郌C[     *
* before_parent	: C[ZbgύXȌC[Zbg *
* after_parent	: C[ZbgύX̏C[Zbg *
***********************************************************/
EXTERN void AddChangeLayerSetHistory(
	const LAYER* change_layer,
	const LAYER* before_parent,
	const LAYER* after_parent
);

EXTERN void FillLayerColor(LAYER* target, uint8 color[3]);

EXTERN void FlipLayerHorizontally(LAYER* target, LAYER* temp);
EXTERN void FlipLayerVertically(LAYER* target, LAYER* temp);

/***************************************
* RasterizeLayer֐                   *
* C[X^CY               *
*                                  *
* target	: X^CY郌C[ *
***************************************/
EXTERN void RasterizeLayer(LAYER* target);

/*****************************
* CreateLayerCopy֐        *
* C[̃Rs[쐬 *
*                        *
* src	: Rs[̃C[ *
* Ԃl                     *
*	쐬C[         *
*****************************/
EXTERN LAYER* CreateLayerCopy(LAYER* src);

/*************************************************
* GetLayerChain֐                              *
* s߂ꂽC[z擾           *
*                                            *
* window	: `̈̏                     *
* num_layer	: C[i[ϐ̃AhX *
* Ԃl                                         *
*	C[z                                 *
*************************************************/
EXTERN LAYER** GetLayerChain(struct _DRAW_WINDOW* window, uint16* num_layer);

/***************************************************
* LayerSetShowChildren֐                         *
* C[Zbg̎qC[\             *
*                                              *
* layer_set	: q\郌C[Zbg           *
* prev		: ֐I̎Ƀ`FbN郌C[ *
***************************************************/
EXTERN void LayerSetShowChildren(LAYER* layer_set, LAYER **prev);

/***************************************************
* LayerSetHideChildren֐                         *
* C[Zbg̎qC[\           *
*                                              *
* layer_set	: q\郌C[Zbg           *
* prev		: ֐I̎Ƀ`FbN郌C[ *
***************************************************/
EXTERN void LayerSetHideChildren(LAYER* layer_set, LAYER **prev);

/***********************************************
* DeleteSelectAreaPixels֐                   *
* I͈͂̃sNZf[^폜           *
*                                          *
* window	: `̈̏                   *
* target	: sNZf[^폜郌C[ *
* selection	: I͈͂Ǘ郌C[       *
***********************************************/
EXTERN void DeleteSelectAreaPixels(
	struct _DRAW_WINDOW* window,
	LAYER* target,
	LAYER* selection
);

/*******************************************************
* AddDeletePixelsHistory֐                           *
* sNZf[^̗f[^ǉ         *
*                                                  *
* tool_name	: sNZf[^Ɏgpc[̖O *
* window	: `̈̏                           *
* min_x		: sNZf[^XWŏl              *
* min_y		: sNZf[^YWŏl              *
* max_x		: sNZf[^XWől              *
* max_y		: sNZf[^YWől              *
*******************************************************/
EXTERN void AddDeletePixelsHistory(
	const char* tool_name,
	struct _DRAW_WINDOW* window,
	LAYER* target,
	int32 min_x,
	int32 min_y,
	int32 max_x,
	int32 max_y
);

/*****************************************************
* CalcLayerAverageRGBA֐                           *
* C[̕σsNZf[^lvZ               *
*                                                *
* target	: σsNZf[^lvZ郌C[ *
* Ԃl                                             *
*	ΏۃC[̕σsNZf[^l               *
*****************************************************/
EXTERN RGBA_DATA CalcLayerAverageRGBA(LAYER* target);

/*****************************************************
* CalcLayerAverageRGBAwithAlpha֐                  *
* llăC[̕σsNZf[^lvZ *
*                                                *
* target	: σsNZf[^lvZ郌C[ *
* Ԃl                                             *
*	ΏۃC[̕σsNZf[^l               *
*****************************************************/
EXTERN RGBA_DATA CalcLayerAverageRGBAwithAlpha(LAYER* target);

EXTERN void OnChangeTextCallBack(GtkTextBuffer* buffer, LAYER* layer);

EXTERN VECTOR_LINE_LAYER* CreateVectorLineLayer(
	struct _LAYER* work,
	VECTOR_LINE* line,
	VECTOR_LAYER_RECTANGLE* rect
);

/*****************************************************
* CreateLayerSetChildButton֐                      *
* C[Zbg̎qC[\E\{^쐬 *
*                                                *
* layer_set	: {^쐬Ώۂ̃C[Zbg         *
* Ԃl                                             *
*	쐬{^EBWFbg                       *
*****************************************************/
EXTERN GtkWidget* CreateLayerSetChildButton(LAYER* layer_set);

/*******************************************************
* FillTextureLayer֐                                 *
* eNX`C[Iꂽp[^[œhׂ *
*                                                  *
* layer		: eNX`C[                       *
* textures	: eNX`̃p[^[                 *
*******************************************************/
EXTERN void FillTextureLayer(LAYER* layer, TEXTURES* textures);

/******************************************
* GetAverageColor֐                     *
* wWӂ̕ϐF擾              *
*                                     *
* target	: F擾郌C[        *
* x			: XW                       *
* y			: YW                       *
* size		: F擾͈            *
* color		: 擾Fi[(4oCg) *
******************************************/
EXTERN void GetAverageColor(LAYER* target, int x, int y, int size, uint8 color[4]);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef _INCLUDED_LAYER_H_
