#ifndef _INCLUDED_VECTOR_H_
#define _INCLUDED_VECTOR_H_

#include <gtk/gtk.h>
#include "types.h"

#ifdef __cplusplus
extern "C" {
#endif

#define VECTOR_LINE_BUFFER_SIZE 32

typedef enum _eVECTOR_LINE_TYPE
{
	VECTOR_LINE_STRAIGHT,
	VECTOR_LINE_BEZIER_OPEN,
	VECTOR_LINE_STRAIGHT_CLOSE,
	VECTOR_LINE_BEZIER_CLOSE
} eVECTOR_LINE_TYPE;

typedef enum _eVECTOR_POINT_FLAGS
{
	VECTOR_POINT_SELECTED = 0x01,
} eVECTOR_POINT_FLAGS;

typedef struct _VECTOR_POINT
{
	uint8 vector_type;
	uint8 flags;
	uint8 color[4];
	gdouble pressure, size;
	gdouble x, y;
} VECTOR_POINT;

typedef enum _eVECTOR_LINE_FLAGS
{
	VECTOR_LINE_ANTI_ALIAS = 0x01,
	VECTOR_LINE_FIX = 0x02,
	VECTOR_LINE_MARKED = 0x04
} eVECTOR_LINE_FLAGS;

typedef struct _VECTOR_LINE_LAYER
{
	int32 x, y, width, height, stride;
	uint8* pixels;
	cairo_t* cairo_p;
	cairo_surface_t* surface_p;
} VECTOR_LINE_LAYER;

/*************************************
* VECTOR_LINE\                  *
* xNgC[̐{̃f[^ *
*************************************/
typedef struct _VECTOR_LINE
{
	uint8 vector_type;			// ̃^Cv
	uint8 flags;				// A`GCAX̃tO
	uint16 num_points;			// _̐
	uint32 buff_size;			// _obt@̃TCY
	gdouble blur;				// {P̊JnItZbg
	gdouble outline_hardness;	// ֊s̍d
	VECTOR_POINT* points;		// _Wz
	VECTOR_LINE_LAYER* layer;	// X^CỸf[^
								// X^CYyp
	// O̐ւ̃|C^
	struct _VECTOR_LINE *prev, *next;
} VECTOR_LINE;

/*********************************************************
* VECTOR_LINE_BASE_DATA\                            *
* xNgC[̐{̊{(f[^op) *
*********************************************************/
typedef struct _VECTOR_LINE_BASE_DATA
{
	uint8 vector_type;			// ̃^Cv
	uint8 flags;				// ɐݒ肳ĂtO
	uint16 num_points;			// _̐
	gdouble blur;				// {P̊JnItZbg
	gdouble outline_hardness;	// ֊s̍d
} VECTOR_LINE_BASE_DATA;

typedef struct _VECTOR_LAYER_RECTANGLE
{
	gdouble min_x, min_y, max_x, max_y;
} VECTOR_LAYER_RECTANGLE;

typedef struct _DIVIDE_LINE_CHANGE_DATA
{
	size_t data_size;
	int8 added;
	int8 line_type;
	uint16 before_num_points;
	uint16 after_num_points;

	int index;
	VECTOR_POINT *before;
	VECTOR_POINT *after;
} DIVIDE_LINE_CHANGE_DATA;

typedef struct _DIVIDE_LINE_ADD_DATA
{
	size_t data_size;
	int8 line_type;
	uint16 num_points;

	int index;
	VECTOR_POINT *after;
} DIVIDE_LINE_ADD_DATA;

typedef struct _DIVIDE_LINE_DATA
{
	int num_change;
	int num_add;
	int change_size;
	int layer_name_length;

	char *layer_name;
	DIVIDE_LINE_CHANGE_DATA *change;
	DIVIDE_LINE_ADD_DATA *add;
} DIVIDE_LINE_DATA;

extern VECTOR_LINE* CreateVectorLine(VECTOR_LINE* prev, VECTOR_LINE* next);

extern void DeleteVectorLine(VECTOR_LINE** line);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef _INCLUDED_VECTOR_H_
