#include <stdlib.h>
#include <string.h>
#include "block.h"

#ifdef __cplusplus
extern "C" {
#endif

void InitializeBlock(
	BLOCK* block,
	int block_size,
	int element_size,
	void (*error_function)(char *)
)
{
	(void)memset(block, 0, sizeof(*block));
	block->block_size = block_size;
	block->element_size = element_size;
	block->error_function = error_function;
}

void ReleaseBlock(BLOCK* block)
{
	while(block->first)
	{
		BLOCK_DATA *next = block->first->next;
		free(block->first);
		block->first = next;
	}
}

void* BlockAppend(BLOCK* block, int num)
{
	void *ret;

	if(block->last == NULL
		|| ((unsigned char*)block->last->current) + (num * block->element_size) > (unsigned char*)block->last->last)
	{
		if(block->last != NULL && block->last->next != NULL)
		{
			block->last = block->last->next;
		}
		else
		{
			BLOCK_DATA *next = (BLOCK_DATA*)malloc(sizeof(BLOCK_DATA) + block->block_size*block->element_size);
			unsigned char *byte_data = (unsigned char*)next;

			if(next == NULL)
			{
				if(block->error_function != NULL)
				{
					block->error_function("Not enough memory!");
					exit(EXIT_FAILURE);
				}
			}
			if(block->last != NULL)
			{
				block->last->next = next;
			}
			else
			{
				block->first = next;
			}
			block->last = next;
			block->last->data = (void*)&byte_data[sizeof(BLOCK_DATA)];
			block->last->current = block->last->data;
			block->last->last = (void*)(((unsigned char*)block->last->current)
				+ block->block_size * block->element_size);
			block->last->next = NULL;
		}
	}

	ret = block->last->current;
	block->last->current = (void*)(((unsigned char*)block->last->current) + (num * block->element_size));
	return ret;
}

void* BlockScanFirst(BLOCK* block)
{
	void *ret;
	block->scan_current_block = block->first;
	if(block->scan_current_block == NULL)
	{
		return NULL;
	}
	block->scan_current_data = block->scan_current_block->data;
	ret = block->scan_current_data;
	block->scan_current_data = (void*)(((unsigned char*)block->scan_current_data) + block->element_size);
	return ret;
}

void* BlockScanNext(BLOCK* block)
{
	void *ret;
	if(block->scan_current_data >= block->scan_current_block->current)
	{
		block->scan_current_block = block->scan_current_block -> next;
		if(block->scan_current_block == NULL)
		{
			return NULL;
		}
		block->scan_current_data = block->scan_current_block->data;
	}
	ret = block->scan_current_data;
	block->scan_current_data = (void*)(((unsigned char*)block->scan_current_data) + block->element_size);
	return ret;
}

void ResetBlock(BLOCK* block)
{
	BLOCK_DATA *b;
	if (block->first == NULL)
	{
		return;
	}
	for (b=block->first; ; b=b->next)
	{
		b->current = b->data;
		if (b == block->last)
		{
			break;
		}
	}
	block->last = block->first;
}

void InitializeDeleteBlock(
	DELETE_BLOCK* block,
	int block_size,
	int element_size,
	void (*err_function)(char *)
)
{
	(void)memset(block, 0, sizeof(*block));
	block->block_size = block_size;
	block->element_size = element_size;
	block->error_function = err_function;
}

void ReleaseDeleteBlock(DELETE_BLOCK* block)
{
	while(block->first)
	{
		DELETE_BLOCK_DATA *next = block->first -> next;
		free(block->first);
		block->first = next;
	}
}

void* DeleteBlockAppend(DELETE_BLOCK* block)
{
	DELETE_BLOCK_ITEM *item;

	if(block->first_free == NULL)
	{
		DELETE_BLOCK_DATA *next = block->first;
		unsigned char *byte_data;
		block->first = (DELETE_BLOCK_DATA*)malloc(sizeof(DELETE_BLOCK_DATA) + (block->block_size-1)*block->element_size);
		byte_data = (unsigned char*)block->first;
		if(block->first == NULL)
		{
			if(block->error_function != NULL)
			{
				block->error_function("Not enough memory!");
				exit(EXIT_FAILURE);
			}
		}
		block->first_free = &block->first->item;
		byte_data = (unsigned char*)block->first_free;
		for(item = block->first_free; (unsigned char*)item < (unsigned char*)block->first_free+(block->block_size-1)*block->element_size;
			item = (DELETE_BLOCK_ITEM*)(((unsigned char*)item) + block->element_size), byte_data += block->element_size)
		{
			item->next_free = (DELETE_BLOCK_ITEM*)&byte_data[block->element_size];
		}
		item->next_free = NULL;
		block->first->next = next;
	}

	item = block->first_free;
	block->first_free = item->next_free;
	return (void*)item;
}

void DeleteBlockRemove(DELETE_BLOCK* block, void* t)
{
	((DELETE_BLOCK_ITEM*)t)->next_free = block->first_free;
	block->first_free = (DELETE_BLOCK_ITEM*) t;
}

#ifdef __cplusplus
}
#endif
