#ifndef _INCLUDED_BLOCK_H_
#define _INCLUDED_BLOCK_H_

#include <stdlib.h>

typedef struct _BLOCK_DATA
{
	void *current, *last;
	struct _BLOCK_DATA *next;
	void* data;
} BLOCK_DATA;

typedef struct _BLOCK
{
	int block_size;
	int element_size;
	BLOCK_DATA *first;
	BLOCK_DATA *last;

	BLOCK_DATA *scan_current_block;
	void *scan_current_data;
	void (*error_function)(char*);
} BLOCK;

typedef union _DELETE_BLOCK_ITEM
{
	void* data;
	union _DELETE_BLOCK_ITEM	*next_free;
} DELETE_BLOCK_ITEM;

typedef struct _DELETE_BLOCK_DATA
{
	struct _DELETE_BLOCK_DATA *next;
	DELETE_BLOCK_ITEM item;
} DELETE_BLOCK_DATA;

typedef struct _DELETE_BLOCK
{
	int block_size;
	int element_size;
	DELETE_BLOCK_DATA *first;
	DELETE_BLOCK_ITEM *first_free;
	void	(*error_function)(char *);
} DELETE_BLOCK;

#ifdef __cplusplus
extern "C" {
#endif

extern void InitializeBlock(
	BLOCK* block,
	int block_size,
	int element_size,
	void (*error_function)(char *)
);

extern void ReleaseBlock(BLOCK* block);

extern void* BlockAppend(BLOCK* block, int num);

extern void* BlockScanFirst(BLOCK* block);

extern void* BlockScanNext(BLOCK* block);

extern void ResetBlock(BLOCK* block);

extern void InitializeDeleteBlock(
	DELETE_BLOCK* block,
	int block_size,
	int element_size,
	void (*err_function)(char *)
);

extern void ReleaseDeleteBlock(DELETE_BLOCK* block);

extern void* DeleteBlockAppend(DELETE_BLOCK* block);

extern void DeleteBlockRemove(DELETE_BLOCK* block, void* t);

#ifdef __cplusplus
}
#endif

#endif	// #ifndef _INCLUDED_BLOCK_H_
