#ifndef _INCLUDED_GRAPH_H_
#define _INCLUDED_GRAPH_H_

#include "block.h"

typedef enum _eGRAPH_TERMINATE_TYPE
{
	GRAPH_TERMINATE_TYPE_SOURCE	= 0,
	GRAPH_TERMINATE_TYPE_SINK	= 1
} eGRAPH_TERMINATE_TYPE; /* terminals */

typedef struct _GRAPH_NODE
{
	struct _GRAPH_ARC *first;
	struct _GRAPH_ARC *parent;
	struct _GRAPH_NODE *next;
	int time_stamp;
	int distance;
	short is_sink;
	short tr_cap;
} GRAPH_NODE;

typedef struct _GRAPH_NODE_POINTER
{
	GRAPH_NODE *pointer;
	struct _GRAPH_NODE_POINTER *next;
} GRAPH_NODE_POINTER;

typedef struct _GRAPH_ARC
{
	GRAPH_NODE *head;
	struct _GRAPH_ARC *next;
	struct _GRAPH_ARC *brother;
	short r_cap;
} GRAPH_ARC;

typedef struct _GRAPH
{
	BLOCK node_block;
	BLOCK arc_block;
	DELETE_BLOCK node_pointer_block;

	void (*error_function)(char*);
	int flow;
	GRAPH_NODE *queue_first[2], *queue_last[2];
	GRAPH_NODE_POINTER *orphan_first, *orphan_last;
	int time_value;
} GRAPH;

extern void DecoupeGraph(
	int* patch_position,
	int width_out,
	int height_out,
	int width_in,
	int height_in,
	int channels,
	unsigned char** rempli,
	unsigned char* image,
	unsigned char* patch,
	unsigned char* coupe_h_here,
	unsigned char* coupe_h_west,
	unsigned char* coupe_v_here,
	unsigned char* coupe_v_north,
	int* compter,
	int make_tileable,
	int invert,
	void (*error_function)(char*)
);

extern void OffsetOptimal(
	int* result_at,
	unsigned char* image,
	unsigned char* patch,
	int width_in,
	int height_in,
	int width_out,
	int height_out,
	int x_patch_position_minimum,
	int y_patch_position_minimum,
	int x_patch_position_maximum,
	int y_patch_position_maximum,
	int channels,
	unsigned char** rempli,
	int tileable
);

extern unsigned char** RempliNew(int x, int y);

extern int CompterRemplis(unsigned char** rempli, int width_out, int height_out);

extern int* PixelRemplir(unsigned char** rempli, int width_out, int height_out, int* result_at);

#endif	// #ifndef _INCLUDED_GRAPH_H_
